"""
Ilab parsers
============

Parsers provided by this module include:

IlabModuleList - command ``/usr/bin/ilab model list``
-----------------------------------------------------
IlabConfigShow - command ``/usr/bin/ilab config show``
------------------------------------------------------
"""
from insights.core import CommandParser, YAMLParser
from insights.core.plugins import parser
from insights.core.exceptions import SkipComponent, ParseException
from insights.specs import Specs


@parser(Specs.ilab_model_list)
class IlabModuleList(CommandParser, list):
    """
    This parser will parse the output of "/usr/bin/ilab model list".

    Sample output from ``/usr/bin/ilab model list``::

        +-----------------------------------+---------------------+---------+
        | Model Name                        | Last Modified       | Size    |
        +-----------------------------------+---------------------+---------+
        | models/prometheus-8x7b-v2-0       | 2024-08-09 13:28:50 |  87.0 GB|
        | models/mixtral-8x7b-instruct-v0-1 | 2024-08-09 13:28:24 |  87.0 GB|
        | models/granite-7b-redhat-lab      | 2024-08-09 14:28:40 |  12.6 GB|
        | models/granite-7b-starter         | 2024-08-09 14:40:35 |  12.6 GB|
        +-----------------------------------+---------------------+---------+

    Examples:
        >>> type(ilab_model_list_info)
        <class 'insights.parsers.ilab.IlabModuleList'>
        >>> ilab_model_list_info[0]["model_name"]
        'models/prometheus-8x7b-v2-0'

    Attributes:
        models (list): list of model names

    """
    def parse_content(self, content):
        if not content:
            raise SkipComponent("Empty")
        if not content[0].startswith("+-"):
            raise ParseException("Unexpected format: %s" % content[0])
        for line in content:
            if line.startswith("|") and "Model Name" not in line:
                split_items = line.split("|")
                if len(split_items) == 5:
                    self.append(
                        {
                            "model_name": split_items[1].strip(),
                            "last_modified": split_items[2].strip(),
                            "size": split_items[3].strip(),
                        }
                    )
                else:
                    raise ParseException("Unexpected format: %s" % line)
        self.models = [m['model_name'] for m in self]


@parser(Specs.ilab_config_show)
class IlabConfigShow(CommandParser, YAMLParser):
    """
    This parser will parse the output of "/usr/bin/ilab config show".

    Sample output from ``/usr/bin/ilab config show``::

        time="2025-04-15T08:23:44Z" level=warning msg="The input device is not a TTY. The --tty and --interactive flags might not work properly"
        # Chat configuration section.
        chat:
          context: default
          logs_dir: /root/.local/share/instructlab/chatlogs
          max_tokens:
          model: /root/.cache/instructlab/models/granite-8b-lab-v1
        evaluate:
          base_branch:
          base_model: /root/.cache/instructlab/models/granite-8b-starter-v1
          mmlu:
            batch_size: auto
            few_shots: 5
        serve:
          backend: vllm
          vllm:
            gpus:
            llm_family: ''
            vllm_args:
            - --tensor-parallel-size
            - '1'

    Examples:
        >>> type(ilab_config_show_info)
        <class 'insights.parsers.ilab.IlabConfigShow'>
        >>> ilab_config_show_info['chat']['logs_dir']
        '/root/.local/share/instructlab/chatlogs'

    Attributes:
        data(dict): The ilab config information
    """
    ignore_lines = ['time=']



"""
Date parsers
============

This module contains the following parsers:

Date - command ``date``
-----------------------
DateUTC - command ``date --utc``
--------------------------------
TimeDateCtlStatus - command ``timedatectl status``
--------------------------------------------------
"""
import six
import sys

from datetime import datetime

from insights.core import CommandParser
from insights.core.exceptions import ParseException, SkipComponent
from insights.core.plugins import parser
from insights.parsers import get_active_lines
from insights.specs import Specs


class DateParseException(Exception):
    pass


class DateParser(CommandParser):
    """Base class implementing shared code."""

    def parse_content(self, content):
        """
        Parses the output of the ``date`` and ``date --utc`` command.

        Sample: Fri Jun 24 09:13:34 CST 2016
        Sample: Fri Jun 24 09:13:34 UTC 2016

        Attributes
        ----------
        datetime: datetime.datetime
            A native datetime.datetime of the parsed date string
        timezone: str
            The string portion of the date string containing the timezone

        Raises:
            DateParseException: Raised if any exception occurs parsing the
            content.
        """
        self.data = get_active_lines(content, comment_char="COMMAND>")[0]
        parts = self.data.split()
        if not len(parts) == 6:
            msg = "Expected six date parts.  Got [%s]"
            raise DateParseException(msg % self.data)
        try:
            self.timezone = parts[4]
            no_tz = ' '.join(parts[:4]) + ' ' + parts[-1]
            self.datetime = datetime.strptime(no_tz, '%a %b %d %H:%M:%S %Y')
        except:
            six.reraise(DateParseException, DateParseException(self.data), sys.exc_info()[2])


@parser(Specs.date)
class Date(DateParser):
    """
    Class to parse ``date`` command output.

    Sample in::

        Fri Jun 24 09:13:34 CST 2016

    Examples:
        >>> from insights.parsers.date import Date
        >>> from insights.tests import context_wrap
        >>> date_content = "Mon May 30 10:49:14 CST 2016"
        >>> shared = {Date: Date(context_wrap(date_content))}
        >>> date_info = shared[Date]
        >>> date_info.data
        'Mon May 30 10:49:14 CST 2016'
        >>> date_info.datetime is not None
        True
        >>> date_info.timezone
        'CST'
    """
    pass


@parser(Specs.date_utc)
class DateUTC(DateParser):
    """
    Class to parse ``date --utc`` command output.

    Sample in::

        Fri Jun 24 09:13:34 UTC 2016

    Examples:
        >>> from insights.parsers.date import DateUTC
        >>> from insights.tests import context_wrap
        >>> date_content = "Mon May 30 10:49:14 UTC 2016"
        >>> shared = {DateUTC: DateUTC(context_wrap(date_content))}
        >>> date_info = shared[DateUTC]
        >>> date_info.data
        'Mon May 30 10:49:14 UTC 2016'
        >>> date_info.datetime
        datetime.datetime(2016, 5, 30, 10, 49, 14)
        >>> date_info.timezone
        'UTC'
    """
    pass


@parser(Specs.timedatectl_status)
class TimeDateCtlStatus(CommandParser, dict):
    """
    Class to parse the ``timedatectl status`` command output.
    It saves the infomartion in each line into a dict.
    Since the colon in all the lines except warning is aligned, every line
    is splited by the same colon index. The key is the lowercase of the first
    part joined by underscore after splitting it by colon, and the value is
    the left part after the colon. If the next line is continue line,
    then append it to the previous key. And also it converts
    the value to datetime format for "Local time", "Universal time" and "RTC time".

    Sample in::

                       Local time: Mon 2022-11-14 23:04:06 PST
                   Universal time: Tue 2022-11-15 07:04:06 UTC
                         RTC time: Tue 2022-11-15 07:04:05
                        Time zone: US/Pacific (PST, -0800)
        System clock synchronized: yes
                      NTP service: active
                  RTC in local TZ: yes
                  Last DST change: DST ended at
                                   Sun 2022-11-06 01:59:59 EDT
                                   Sun 2022-11-06 01:00:00 EST
                  Next DST change: DST begins (the clock jumps one hour forward) at
                                   Sun 2023-03-12 01:59:59 EST
                                   Sun 2023-03-12 03:00:00 EDT

        Warning: The system is configured to read the RTC time in the local time zone.
                This mode cannot be fully supported. It will create various problems
                with time zone changes and daylight saving time adjustments. The RTC
                time is never updated, it relies on external facilities to maintain it.
                If at all possible, use RTC in UTC by calling
                'timedatectl set-local-rtc 0'.

    Raises:
        DateParseException: when the datetime in "Local time", "Universal time",
                            "RTC time" are not in expected format.
        ParseException: when the colon in each line except warning is not aligned.

    Examples:
        >>> ctl_info['ntp_service']
        'active'
        >>> ctl_info['system_clock_synchronized']
        'yes'
        >>> ctl_info['local_time']
        datetime.datetime(2022, 11, 14, 23, 4, 6)
    """
    date_format = '%a %Y-%m-%d %H:%M:%S'

    # unify the different names in rhel7 and rhel8
    key_mapping = {
        'ntp_synchronized': 'system_clock_synchronized'
    }

    def parse_content(self, content):
        dict_key = None
        warning_start = False
        non_blank_line = None
        for line in content:
            if line.strip():
                non_blank_line = line
                break
        if non_blank_line is None:
            raise SkipComponent('No data in the output.')
        try:
            colon_index = non_blank_line.index(':')
        except ValueError:
            raise ParseException('No colon found, the line %s is not in expected format.' % line)
        for line in content:
            if not line.strip():
                continue
            if line[colon_index] == ':':
                key = line[:colon_index].strip()
                value = line[colon_index + 1:].strip()
                dict_key = '_'.join(key.lower().split())
                if dict_key in ['local_time', 'universal_time', 'rtc_time']:
                    if dict_key == 'rtc_time':
                        final_val = value
                    else:
                        final_val = value.rsplit(None, 1)[0]  # remove tz info
                    try:
                        self[dict_key] = datetime.strptime(final_val, self.date_format)
                    except Exception:
                        six.reraise(DateParseException, DateParseException(value), sys.exc_info()[2])
                else:
                    if dict_key in self.key_mapping:
                        self[self.key_mapping[dict_key]] = value
                    else:
                        self[dict_key] = value
            elif not line[:colon_index].strip():
                # this line is also the content of the previous key
                self[dict_key] += ' ' + line.strip()
            elif "Warning: " in line:
                warning_start = True
                self['warning'] = line.split(':')[1].strip()
            else:
                if warning_start:
                    self['warning'] += ' ' + line.strip()
                else:
                    raise ParseException('Unexpected format of line %s.' % line)



"""
db2 - db2 database information
==============================
Parsers included in this module are:

Db2ls - command ``db2ls -a -c``
-------------------------------

Db2DatabaseConfiguration - Command ``/usr/sbin/runuser -l  <user name>  -c 'db2 get database configuration for %s'``
--------------------------------------------------------------------------------------------------------------------

Db2DatabaseManager - Command ``/usr/sbin/runuser -l  <user name>  -c 'db2 get dbm cfg'``
----------------------------------------------------------------------------------------
"""

from insights.core import CommandParser
from insights.core.exceptions import SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.db2ls_a_c)
class Db2ls(CommandParser, list):
    """
    Parse the output of `db2ls -a -c` into a list of dictionaries.

    Typical output of this command::

        #PATH:VRMF:FIXPACK:SPECIAL:INSTALLTIME:INSTALLERUID
        /opt/ibm/db2/V11.5:11.5.6.0:0 ::Fri Jan 14 20:20:07 2022 CST :0
        /opt/ibm/db2/V11.5_01:11.5.7.0:0 ::Fri Feb 11 10:34:51 2022 CST :0

    Examples:
        >>> type(db2ls)
        <class 'insights.parsers.db2.Db2ls'>
        >>> len(db2ls)
        2
        >>> db2ls[0]['PATH']
        '/opt/ibm/db2/V11.5'
        >>> db2ls[1]['PATH']
        '/opt/ibm/db2/V11.5_01'
        >>> db2ls[1]['VRMF']
        '11.5.7.0'
        >>> db2ls[1]['INSTALLERUID']
        '0'
    """

    def parse_content(self, content):
        if not content:
            raise SkipComponent("Empty.")
        keys = []
        for line in content:
            if not keys and line.startswith('#PATH'):
                keys = [i.strip('#') for i in line.split(':')]
                continue
            line_splits = [i.strip() for i in line.split(':', 4)]
            line_splits.extend([i.strip(': ') for i in line_splits.pop(-1).rsplit(':', 1)])
            self.append(dict(zip(keys, line_splits)))
        if len(self) == 0:
            raise SkipComponent('Nothing to parse.')


@parser(Specs.db2_database_configuration)
class Db2DatabaseConfiguration(CommandParser, dict):
    """
    Parse the output of the ``/usr/sbin/runuser -l  <user name>  -c 'db2 get database configuration for %s'`` command.

    Attributes:
        default (dict): the database configuration details

    Sample Input::

               Database Configuration for Database TESTD1

        Database configuration release level                    = 0x1500
        Database release level                                  = 0x1500

        Update to database level pending                        = NO (0x0)
        Database territory                                      = US
        Database code page                                      = 1208

    Examples:
        >>> db2databaseconfiguration["Database configuration release level"]
        '0x1500'
        >>> db2databaseconfiguration["user"]
        'dbp'
    """

    def parse_content(self, content):
        if not content:
            raise SkipComponent("The result is empty")

        result = {}

        for line in content:
            if "=" in line:
                key, value = line.split("=")
                strip_key = key.strip()
                if "(" in strip_key and strip_key[-1] == ")":
                    key_name = strip_key.rsplit(")", 1)[0].rsplit("(", 1)[-1].strip()
                    if key_name.isupper():
                        result[key_name] = value.strip()
                    else:
                        result[strip_key] = value.strip()
                else:
                    result[strip_key] = value.strip()

        if not result:
            raise SkipComponent("The format is incorrect")

        result["user"] = self.file_path.split("_")[3].strip()
        result["db_name"] = self.file_path.split("_")[-1].strip()
        self.update(result)


@parser(Specs.db2_database_manager)
class Db2DatabaseManager(CommandParser, dict):
    """
    Parse the output of the ``/usr/sbin/runuser -l  <user name>  -c 'db2 get dbm cfg'`` command.

    Attributes:
        default (dict): the database management details

    Sample Input::


              Database Manager Configuration

        Node type = Enterprise Server Edition with local and remote clients

        Database manager configuration release level            = 0x1500

        CPU speed (millisec/instruction)             (CPUSPEED) = 3.306409e-07
        Communications bandwidth (MB/sec)      (COMM_BANDWIDTH) = 1.000000e+02

        Max number of concurrently active databases     (NUMDB) = 32
        Federated Database System Support           (FEDERATED) = NO

    Examples:
        >>> db2databasemanager["NUMDB"]
        '32'
        >>> db2databasemanager["user"]
        'dbp'
    """

    def parse_content(self, content):
        if not content:
            raise SkipComponent("The result is empty")

        result = {}

        for line in content:
            if "=" in line:
                key, value = line.split("=")
                strip_key = key.strip()
                if "(" in strip_key and strip_key[-1] == ")":
                    key_name = strip_key.rsplit(")", 1)[0].rsplit("(", 1)[-1].strip()
                    if key_name.isupper():
                        result[key_name] = value.strip()
                    else:
                        result[strip_key] = value.strip()
                else:
                    result[strip_key] = value.strip()

        if not result:
            raise SkipComponent("The format is incorrect")

        result["user"] = self.file_path.split("_")[3]

        self.update(result)



import doctest

from insights.parsers import insights_client_conf
from insights.tests import context_wrap

CLIENT_CONF = """
[insights-client]
# Change log level, valid options DEBUG, INFO, WARNING, ERROR, CRITICAL. Default DEBUG
loglevel=INFO
# Log each line executed
trace=False
# Attempt to auto configure with Satellite server
auto_config=True
# Automatically update the dynamic configuration
auto_update=False
# Obfuscate IP addresses
obfuscate=False
""".strip()

BASIC_INSIGHTS_CLIENT = """
{"username_set": true, "pass_set": true}
""".strip()


def test_doc_examples():
    failed_count, tests = doctest.testmod(
        insights_client_conf,
        globs={'conf': insights_client_conf.InsightsClientConf(context_wrap(CLIENT_CONF)),
               'basic_conf': insights_client_conf.BasicAuthInsightsClient(context_wrap(BASIC_INSIGHTS_CLIENT))
               }
    )
    assert failed_count == 0


def test_basic_insights_client():
    ret = insights_client_conf.BasicAuthInsightsClient(context_wrap(BASIC_INSIGHTS_CLIENT))
    assert "username_set" in ret
    assert "pass_set" in ret


def test_insights_client_conf():
    conf = insights_client_conf.InsightsClientConf(context_wrap(CLIENT_CONF))
    assert conf is not None
    assert list(conf.sections()) == ['insights-client']
    assert conf.has_option('insights-client', 'auto_update')
    assert not conf.has_option('yabba', 'dabba_do')
    assert conf.get('insights-client', 'auto_update') == 'False'



#!/usr/bin/env python
"""
Hostname Release
================

This is a simple rule and can be run against the local host
using the following command::

    $ insights-run -p examples.rules.hostname_rel

or from the examples/rules directory::

    $ ./hostname_rel.py
"""
from insights.core.plugins import make_fail, make_pass, rule
from insights.parsers.hostname import Hostname
from insights.parsers.redhat_release import RedhatRelease

ERROR_KEY_1 = "RELEASE_IS_RHEL"
ERROR_KEY_2 = "RELEASE_IS_NOT_RECOGNIZED"
ERROR_KEY_3 = "RELEASE_CANNOT_BE_DETERMINED"

CONTENT = {
    ERROR_KEY_1: "This release is RHEL\nHostname: {{ hostname }}\nRelease: {{ release }}",
    ERROR_KEY_2: "This release is not RHEL\nHostname: {{ hostname }}\nRelease: {{ release }}",
    ERROR_KEY_3: "This release is not RHEL\nHostname: {{ hostname }}\nRelease: not present"
}


@rule(Hostname, [RedhatRelease])
def report(hostname, release):
    if release and release.is_rhel:
        return make_pass(ERROR_KEY_1,
                         hostname=hostname.fqdn,
                         release=release.version)
    elif release:
        return make_fail(ERROR_KEY_2,
                         hostname=hostname.fqdn,
                         release=release.raw)
    else:
        return make_fail(ERROR_KEY_3, hostname=hostname.fqdn)


if __name__ == "__main__":
    from insights import run
    run(report, print_summary=True)



"""
Custom datasource for gathering a list of encrypted LUKS block devices and their properties.
"""

import re

from insights.components.cryptsetup import HasCryptsetupWithTokens, HasCryptsetupWithoutTokens
from insights.core.context import HostContext
from insights.core.exceptions import SkipComponent
from insights.core.plugins import datasource
from insights.core.spec_factory import DatasourceProvider, foreach_execute
from insights.parsers.blkid import BlockIDInfo
from insights.specs import Specs


@datasource(BlockIDInfo, HostContext)
def luks_block_devices(broker):
    """
    This datasource provides a list of LUKS encrypted device.

    Sample data returned::

        ['/dev/sda', '/dev/nvme0n1p3']

    Returns:
        list: List of the LUKS encrypted block devices.

    Raises:
        SkipComponent: When there is not any LUKS encrypted block device on the
        system.
    """

    block_id = broker[BlockIDInfo]
    if block_id:
        devices = block_id.filter_by_type("crypto_LUKS")
        if devices:
            return sorted(map(lambda x: x["NAME"], devices))

    raise SkipComponent


class LocalSpecs(Specs):
    """Local specs used only by LUKS_data_sources datasource."""

    cryptsetup_luks_dump_token_commands = foreach_execute(
        luks_block_devices,
        "/usr/sbin/cryptsetup luksDump --disable-external-tokens %s",
        deps=[luks_block_devices, HasCryptsetupWithTokens],
    )
    cryptsetup_luks_dump_commands = foreach_execute(
        luks_block_devices,
        "/usr/sbin/cryptsetup luksDump %s",
        deps=[luks_block_devices, HasCryptsetupWithoutTokens],
    )


def line_indentation(line):
    """
    Compute line indentation level

    Arguments:
        line(str): The whole line

    Returns:
        int: the number of spaces the line is indentated by
    """
    line = line.replace("\t", " " * 8)
    return len(line) - len(line.lstrip())


def filter_token_lines(lines):
    """
    Filter out token descriptions to keep just the Keyslot filed

    Arguments:
        lines(list): List of lines of the luksDump output

    Returns:
        list: The original lines, except the tokens section only contains only token name and associated keyslot
    """
    in_tokens = False
    remove_indices = []

    for i, line in enumerate(lines):
        if line == "Tokens:":
            in_tokens = True
            continue

        if in_tokens and line_indentation(line) == 0:
            in_tokens = False

        if not in_tokens or line_indentation(line) == 2 or line.startswith("\tKeyslot:"):
            continue

        remove_indices.append(i)

    return [i for j, i in enumerate(lines) if j not in remove_indices]


@datasource(
    HostContext,
    [LocalSpecs.cryptsetup_luks_dump_token_commands, LocalSpecs.cryptsetup_luks_dump_commands],
)
def luks_data_sources(broker):
    """
    This datasource provides the output of 'cryptsetup luksDump' command for
    every LUKS encrypted device on the system. The digest and salt fields are
    filtered out as they can be potentially sensitive.

    Returns:
        list: List of outputs of the cryptsetup luksDump command.

    Raises:
        SkipComponent: When there is not any LUKS encrypted block device on the
        system.
    """
    datasources = []

    commands = []
    if LocalSpecs.cryptsetup_luks_dump_token_commands in broker:
        commands.extend(broker[LocalSpecs.cryptsetup_luks_dump_token_commands])
    if LocalSpecs.cryptsetup_luks_dump_commands in broker:
        commands.extend(broker[LocalSpecs.cryptsetup_luks_dump_commands])

    for command in commands:
        lines_without_tokens = filter_token_lines(command.content)

        regex = re.compile(
            r'[\t ]*(MK digest:|MK salt:|Salt:|Digest:)(\s*([a-z0-9][a-z0-9] ){16}\n)*(\s*([a-z0-9][a-z0-9] )+\n)?',
            flags=re.IGNORECASE,
        )
        filtered_content = regex.sub("", "\n".join(lines_without_tokens) + "\n")

        relative_path = "insights_datasources/" + command.cmd.replace("/", ".").replace(" ", "_")
        datasources.append(
            DatasourceProvider(
                content=filtered_content,
                relative_path=relative_path,
            )
        )

    if datasources:
        return datasources

    raise SkipComponent



import pytest

from mock.mock import Mock

from insights.core.exceptions import SkipComponent
from insights.specs.datasources.aws import LocalSpecs, aws_imdsv2_token


TOKEN = "1234567890\n"

TOKEN_INVALID = "Warning: /root/.curlrc:1: warning: 'proxy' requires parameter"


def test_aws_imdsv2_token():
    input_spec = Mock()
    input_spec.content = [TOKEN, ]
    broker = {LocalSpecs.aws_imdsv2_token: input_spec}
    results = aws_imdsv2_token(broker)
    assert results == TOKEN.strip()


def test_aws_imdsv2_token_exp():
    input_spec = Mock()
    input_spec.content = []
    broker = {LocalSpecs.aws_imdsv2_token: input_spec}
    with pytest.raises(Exception) as ex:
        aws_imdsv2_token(broker)
    assert "Unexpected" in str(ex)

    input_spec = Mock()
    input_spec.content = ["  ", ]
    broker = {LocalSpecs.aws_imdsv2_token: input_spec}
    with pytest.raises(SkipComponent) as ex:
        aws_imdsv2_token(broker)

    input_spec = Mock()
    input_spec.content = [TOKEN_INVALID, ]
    broker = {LocalSpecs.aws_imdsv2_token: input_spec}
    with pytest.raises(SkipComponent) as ex:
        aws_imdsv2_token(broker)



from insights.parsers.ip import IpLinkInfo
from insights.parsers.netstat import Netstat_I
from insights.util import keys_in
from insights.combiners.netstat import NetworkStats
from insights.tests import context_wrap

NETSTAT_I = """
Kernel Interface table
Iface       MTU Met    RX-OK RX-ERR RX-DRP RX-OVR    TX-OK TX-ERR TX-DRP TX-OVR Flg
bond0      1500   0   845265      0      0      0     1753      0      0      0 BMmRU
bond1      1500   0   842447      0      0      0     4233      0      0      0 BMmRU
eth0       1500   0   422518      0      0      0     1703      0      0      0 BMsRU
eth1       1500   0   422747      0      0      0       50      0      0      0 BMsRU
eth2       1500   0   421192      0      0      0     3674      0      0      0 BMsRU
eth3       1500   0   421255      0      0      0      559      0      0      0 BMsRU
lo        65536   0        0      0      0      0        0      0      0      0 LRU
""".strip()

IP_S_LINK = """
1: enp0s3: <BROADCAST,MULTICAST,UP,LOWER_UP> mtu 1500 qdisc pfifo_fast state UP mode DEFAULT qlen 1000
    link/ether 08:00:27:4a:c5:ef brd ff:ff:ff:ff:ff:ff
    RX: bytes  packets  errors  dropped overrun mcast
    1113685    2244     0       0       0       0
    TX: bytes  packets  errors  dropped carrier collsns
    550754     1407     0       0       0       0
2: lo: <LOOPBACK,UP,LOWER_UP> mtu 65536 qdisc noqueue state UNKNOWN mode DEFAULT
    link/loopback 00:00:00:00:00:00 brd 00:00:00:00:00:00
    RX: bytes  packets  errors  dropped overrun mcast
    884        98       0       0       0       0
    TX: bytes  packets  errors  dropped carrier collsns
    884        100       0       0       0       0
3: enp0s8: <BROADCAST,MULTICAST,UP,LOWER_UP> mtu 1500 qdisc pfifo_fast state UP mode DEFAULT qlen 1000
    link/ether 08:00:27:db:86:9e brd ff:ff:ff:ff:ff:ff
    RX: bytes  packets  errors  dropped overrun mcast
    0          6        0       0       0       0
    TX: bytes  packets  errors  dropped carrier collsns
    0          4        0       0       0       0
4: enp0s9: <BROADCAST,UP,MULTICAST> mtu 1500 qdisc pfifo_fast state UP mode DEFAULT qlen 1000
    link/ether 08:00:27:a6:bd:65 brd ff:ff:ff:ff:ff:ff
    RX: bytes  packets  errors  dropped overrun mcast
    0          8        0       0       0       0
    TX: bytes  packets  errors  dropped carrier collsns
    0          12        0       0       0       0
""".strip()

IP_S_LINK_2 = """
1: lo: <LOOPBACK,UP,LOWER_UP> mtu 65536 qdisc noqueue state UNKNOWN mode DEFAULT group default qlen 1000
    link/loopback 00:00:00:00:00:00 brd 00:00:00:00:00:00
    RX: bytes  packets  errors  dropped overrun mcast
    1736       20       0       0       0       0
    TX: bytes  packets  errors  dropped carrier collsns
    1736       20       0       0       0       0
2: enp0s31f6: <NO-CARRIER,BROADCAST,MULTICAST,UP> mtu 1500 qdisc fq_codel state DOWN mode DEFAULT group default qlen 1000
    link/ether c8:5b:76:3f:14:d5 brd ff:ff:ff:ff:ff:ff
    RX: bytes  packets  errors  dropped overrun mcast
    0          0        0       0       0       0
    TX: bytes  packets  errors  dropped carrier collsns
    0          0        0       0       0       0
3: wlp4s0: <BROADCAST,MULTICAST,UP,LOWER_UP> mtu 1500 qdisc mq state UP mode DORMANT group default qlen 1000
    link/ether e4:a4:71:ae:70:b3 brd ff:ff:ff:ff:ff:ff
    RX: bytes  packets  errors  dropped overrun mcast
    96421231   90178    0       0       0       0
    TX: bytes  packets  errors  dropped carrier collsns
    7341914    51363    0       0       0       0
4: virbr0: <NO-CARRIER,BROADCAST,MULTICAST,UP> mtu 1500 qdisc noqueue state DOWN mode DEFAULT group default qlen 1000
    link/ether 52:54:00:97:c0:bf brd ff:ff:ff:ff:ff:ff
    RX: bytes  packets  errors  dropped overrun mcast
    0          0        0       0       0       0
    TX: bytes  packets  errors  dropped carrier collsns
    0          0        0       0       0       0
5: virbr0-nic: <BROADCAST,MULTICAST> mtu 1500 qdisc fq_codel master virbr0 state DOWN mode DEFAULT group default qlen 1000
    link/ether 52:54:00:97:c0:bf brd ff:ff:ff:ff:ff:ff
    RX: bytes  packets  errors  dropped overrun mcast
    0          0        0       0       0       0
    TX: bytes  packets  errors  dropped carrier collsns
    0          0        0       0       0       0
6: tun0: <POINTOPOINT,MULTICAST,NOARP,UP,LOWER_UP> mtu 1360 qdisc fq_codel state UNKNOWN mode DEFAULT group default qlen 100
    link/none
    RX: bytes  packets  errors  dropped overrun mcast
    14991388   21827    0       0       0       0
    TX: bytes  packets  errors  dropped carrier collsns
    1448430    22760    0       0       0       0
""".strip()

NETSTAT_I_2 = """
Kernel Interface table
Iface      MTU    RX-OK RX-ERR RX-DRP RX-OVR    TX-OK TX-ERR TX-DRP TX-OVR Flg
enp0s31f  1500        0      0      0 0             0      0      0      0 BMU
lo       65536       20      0      0 0            20      0      0      0 LRU
tun0      1360    21898      0      0 0         22839      0      0      0 MOPRU
virbr0    1500        0      0      0 0             0      0      0      0 BMU
wlp4s0    1500    90309      0      0 0         51481      0      0      0 BMRU
""".strip()

NETSTAT_I_NO = """
""".strip()

IP_S_LINK_NO = """
""".strip()


def test_ip_data_Link():
    link_info = IpLinkInfo(context_wrap(IP_S_LINK))
    if_list = link_info.active

    assert len(if_list) == 4
    assert keys_in(["lo", "enp0s3", "enp0s8", "enp0s9"], if_list)

    assert sorted(link_info.active) == sorted(['lo', 'enp0s3', 'enp0s8', 'enp0s9'])

    lo = link_info["lo"]
    assert lo["mac"] == "00:00:00:00:00:00"
    assert lo["flags"] == ["LOOPBACK", "UP", "LOWER_UP"]
    assert lo["type"] == "loopback"
    assert lo["mtu"] == 65536
    assert lo["rx_packets"] == 98
    assert lo["tx_packets"] == 100
    assert lo["index"] == 2

    enp0s3 = link_info["enp0s3"]
    assert enp0s3["mac"] == "08:00:27:4a:c5:ef"
    assert enp0s3["flags"] == ["BROADCAST", "MULTICAST", "UP", "LOWER_UP"]
    assert enp0s3["type"] == "ether"
    assert enp0s3["mtu"] == 1500
    assert enp0s3["rx_packets"] == 2244
    assert enp0s3["tx_packets"] == 1407
    assert enp0s3["index"] == 1


def test_get_netstat_i():
    netstat = Netstat_I(context_wrap(NETSTAT_I))
    nstat = NetworkStats(netstat, None)
    result = nstat.group_by_iface
    assert len(result) == 7
    assert result["bond0"] == {
        "MTU": "1500", "Met": "0", "RX-OK": "845265", "RX-ERR": "0",
        "RX-DRP": "0", "RX-OVR": "0", "TX-OK": "1753", "TX-ERR": "0",
        "TX-DRP": "0", "TX-OVR": "0", "Flg": "BMmRU"
    }
    assert result["eth0"] == {
        "MTU": "1500", "Met": "0", "RX-OK": "422518", "RX-ERR": "0",
        "RX-DRP": "0", "RX-OVR": "0", "TX-OK": "1703", "TX-ERR": "0",
        "TX-DRP": "0", "TX-OVR": "0", "Flg": "BMsRU"
    }


def test_combined():
    context = context_wrap(NETSTAT_I)
    nstat = Netstat_I(context)
    networkstats = NetworkStats(nstat, None)
    assert networkstats.data[0]["Iface"] == "bond0"
    result = networkstats.group_by_iface
    assert len(result) == 7
    assert result["bond0"] == {
        "MTU": "1500", "Met": "0", "RX-OK": "845265", "RX-ERR": "0",
        "RX-DRP": "0", "RX-OVR": "0", "TX-OK": "1753", "TX-ERR": "0",
        "TX-DRP": "0", "TX-OVR": "0", "Flg": "BMmRU"
    }

    context = context_wrap(IP_S_LINK)
    linkinfo = IpLinkInfo(context)
    networkstats = NetworkStats(None, linkinfo)
    result = networkstats.group_by_iface
    # the order of this structure is a cpython implementation detail
    # assert networkstats.data[0]["Iface"] == "lo"
    assert len(result) == 4
    assert result["lo"] == {
        'RX-OK': 98, 'TX-OK': 100, 'MTU': 65536, 'RX-ERR': 0,
        'TX-DRP': 0, 'TX-ERR': 0, 'RX-DRP': 0, 'RX-OVR': 0, 'Flg': 'LRU'
    }
    assert result["enp0s8"] == {
        'RX-OK': 6, 'TX-DRP': 0, 'TX-OK': 4, 'MTU': 1500,
        'RX-ERR': 0, 'TX-ERR': 0, 'RX-DRP': 0, 'RX-OVR': 0, 'Flg': 'BMRU'
    }

    context = context_wrap(NETSTAT_I_2)
    nstat = Netstat_I(context)
    networkstats = NetworkStats(nstat, None)
    result = networkstats.group_by_iface
    assert len(result) == 5

    context = context_wrap(IP_S_LINK_NO)
    linkinfo = IpLinkInfo(context)
    networkstats = NetworkStats(None, linkinfo)
    result = networkstats.group_by_iface
    assert len(result) == 0

    context = context_wrap(IP_S_LINK_2)
    linkinfo = IpLinkInfo(context)
    networkstats = NetworkStats(None, linkinfo)
    result = networkstats.group_by_iface
    assert len(result) == 6



import json
from insights.client.config import InsightsConfig
from insights.client.connection import InsightsConnection
from mock.mock import patch, mock_open, ANY
from pytest import raises


class MockSession(object):
    def __init__(self):
        self.status_code = None
        self.text = None
        self.content = '{"display_name": "test"}'

    def get(self, url=None, timeout=None, headers=None, data=None):
        return MockResponse(self.status_code, self.text, self.content, None)

    def post(self, url=None, timeout=None, headers=None, data=None, files=None):
        return MockResponse(self.status_code, self.text, self.content, None)

    def put(self, url=None, timeout=None, headers=None, data=None):
        return MockResponse(self.status_code, self.text, None, None)


class MockResponse(object):
    def __init__(self, expected_status, expected_text, expected_content, expected_reason):
        self.status_code = expected_status
        self.text = expected_text
        self.content = expected_content
        self.reason = expected_reason


def mock_init_session(obj):
    return MockSession()


def mock_get_proxies(obj):
    return


def mock_machine_id():
    return 'XXXXXXXX'


class MockMagic():
    def __init__(*args):
        pass

    def load(self):
        return None

    def file(self, *args):
        return 'application/gzip'


def test_config_conflicts():
    '''
    Ensure --payload requires --content-type
    '''
    with raises(ValueError) as v:
        InsightsConfig(payload='aaa')
    assert str(v.value) == '--payload requires --content-type'


@patch('insights.client.connection.InsightsConnection._init_session',
       mock_init_session)
@patch('insights.client.connection.InsightsConnection.get_proxies',
       mock_get_proxies)
def test_upload_urls():
    '''
    Ensure upload urls are defined correctly
    '''
    # legacy default
    conf = InsightsConfig()
    c = InsightsConnection(conf)
    assert c.upload_url == c.base_url + '/uploads'

    # plaform implied
    conf = InsightsConfig(legacy_upload=False)
    c = InsightsConnection(conf)
    assert c.upload_url == c.base_url + '/ingress/v1/upload'

    # explicitly configured
    conf = InsightsConfig(upload_url='BUNCHANONSENSE')
    c = InsightsConnection(conf)
    assert c.upload_url == 'BUNCHANONSENSE'


@patch('insights.client.connection.InsightsUploadConf.get_rm_conf', return_value={})
@patch("insights.client.connection.InsightsConnection._legacy_upload_archive")
@patch("insights.client.connection.get_canonical_facts", return_value={'test': 'facts'})
@patch('insights.client.connection.InsightsConnection.post')
@patch("insights.client.connection.open", new_callable=mock_open)
def test_payload_upload(op, post, c, _legacy_upload_archive, rm_conf):
    '''
    Ensure a payload upload occurs with the right URL and params
    '''
    conf = InsightsConfig(legacy_upload=False)
    c = InsightsConnection(conf)
    c.upload_archive('testp', 'testct', None)
    post.assert_called_with(
        c.base_url + '/ingress/v1/upload',
        files={
            'file': ('testp', ANY, 'testct'),  # ANY = return call from mocked open(), acts as filepointer here
            'metadata': json.dumps({
                'test': 'facts',
                'branch_info': {'remote_branch': -1, 'remote_leaf': -1},
                'satellite_id': -1,
            })},
        headers={})
    _legacy_upload_archive.assert_not_called()


@patch('insights.contrib.magic.open', MockMagic)
@patch('insights.client.connection.generate_machine_id', mock_machine_id)
@patch("insights.client.connection.get_canonical_facts", return_value={'test': 'facts'})
@patch('insights.client.connection.InsightsConnection.post')
@patch("insights.client.connection.open", new_callable=mock_open)
def test_legacy_upload(op, post, c):
    '''
    Ensure an Insights collected tar upload to legacy occurs with the right URL and params
    '''
    conf = InsightsConfig()
    c = InsightsConnection(conf)
    c.upload_archive('testp', 'testct', None)
    post.assert_called_with(
        c.base_url + '/uploads/XXXXXXXX',
        files={
            'file': ('testp', ANY, 'application/gzip')},  # ANY = return call from mocked open(), acts as filepointer here
        headers={'x-rh-collection-time': 'None'})


@patch("insights.client.connection.InsightsConnection._legacy_upload_archive")
def test_legacy_upload_func(_legacy_upload_archive):
    '''
    Ensure the _legacy_upload_archive path is used
    '''
    conf = InsightsConfig()
    c = InsightsConnection(conf)
    c.upload_archive('testp', 'testct', None)
    _legacy_upload_archive.assert_called_with('testp', None)



# -*- coding: UTF-8 -*-

from contextlib import contextmanager
import os
from shutil import rmtree
from insights.client.phase.v1 import post_update
from mock.mock import patch, MagicMock
from pytest import raises


def patch_insights_config(old_function):
    patcher = patch("insights.client.phase.v1.InsightsConfig",
                    **{"return_value.load_all.return_value.status": False,
                       "return_value.load_all.return_value.unregister": False,
                       "return_value.load_all.return_value.offline": False,
                       "return_value.load_all.return_value.enable_schedule": False,
                       "return_value.load_all.return_value.disable_schedule": False,
                       "return_value.load_all.return_value.analyze_container": False,
                       "return_value.load_all.return_value.display_name": False,
                       "return_value.load_all.return_value.register": False,
                       "return_value.load_all.return_value.legacy_upload": False,
                       "return_value.load_all.return_value.diagnosis": None,
                       "return_value.load_all.return_value.payload": None,
                       "return_value.load_all.return_value.list_specs": False,
                       "return_value.load_all.return_value.show_results": False,
                       "return_value.load_all.return_value.check_results": False,
                       "return_value.load_all.return_value.no_upload": False,
                       "return_value.load_all.return_value.module": False})
    return patcher(old_function)

# DRY this at some point... for the love of god


@patch("insights.client.phase.v1.isfile", side_effect=[True])
@patch("insights.client.phase.v1.InsightsClient")
@patch_insights_config
def test_post_update_no_options_registered(insights_config, insights_client, _isfile):
    """
    Client run with no options.
        If registered, exit with 0 exit code (don't kill parent)
    """
    with raises(SystemExit) as exc_info:
        post_update()
    assert exc_info.value.code == 0
    insights_client.return_value.get_machine_id.assert_called_once()
    insights_client.return_value.clear_local_registration.assert_not_called()
    insights_client.return_value.set_display_name.assert_not_called()


@patch("insights.client.phase.v1.isfile", side_effect=[False])
@patch("insights.client.phase.v1.InsightsClient")
@patch_insights_config
def test_post_update_no_options_unregistered(insights_config, insights_client, _isfile):
    """
    Client run with no options.
        If unregistered, exit with 101 exit code (kill parent)
    """
    with raises(SystemExit) as exc_info:
        post_update()
    assert exc_info.value.code == 101
    insights_client.return_value.clear_local_registration.assert_not_called()
    insights_client.return_value.set_display_name.assert_not_called()


@patch("insights.client.phase.v1.isfile", side_effect=[False])
@patch("insights.client.phase.v1.InsightsClient")
@patch_insights_config
def test_post_update_no_options_err_reg_check(insights_config, insights_client, _isfile):
    """
    Client run with no options.
        If registration check fails, exit with 101 exit code
    """
    with raises(SystemExit) as exc_info:
        post_update()
    assert exc_info.value.code == 101
    insights_client.return_value.clear_local_registration.assert_not_called()
    insights_client.return_value.set_display_name.assert_not_called()


@patch("insights.client.phase.v1.isfile", side_effect=[True])
@patch("insights.client.phase.v1.InsightsClient")
@patch_insights_config
def test_post_update_check_status_registered(insights_config, insights_client, _isfile):
    """
    Just check status.
        If registered, exit with 100 exit code (kill parent)
    """
    insights_config.return_value.load_all.return_value.status = True
    with raises(SystemExit) as exc_info:
        post_update()
    assert exc_info.value.code == 100
    insights_client.return_value.clear_local_registration.assert_not_called()
    insights_client.return_value.set_display_name.assert_not_called()


@patch("insights.client.phase.v1.isfile", side_effect=[False])
@patch("insights.client.phase.v1.InsightsClient")
@patch_insights_config
def test_post_update_check_status_unregistered(insights_config, insights_client, _isfile):
    """
    Just check status.
        If unregistered, exit with 101 exit code (kill parent)
    """
    insights_config.return_value.load_all.return_value.status = True
    with raises(SystemExit) as exc_info:
        post_update()
    assert exc_info.value.code == 101
    insights_client.return_value.clear_local_registration.assert_not_called()
    insights_client.return_value.set_display_name.assert_not_called()


@patch("insights.client.phase.v1.isfile", side_effect=[True])
@patch("insights.client.phase.v1.get_scheduler")
@patch("insights.client.phase.v1.InsightsClient")
@patch_insights_config
def test_post_update_register_registered(insights_config, insights_client, get_scheduler, _isfile):
    """
    Client run with --register.
        If registered, exit with 0 exit code
    """
    insights_config.return_value.load_all.return_value.register = True
    with raises(SystemExit) as exc_info:
        post_update()
    assert exc_info.value.code == 0
    insights_client.return_value.get_machine_id.assert_called_once()
    insights_client.return_value.clear_local_registration.assert_not_called()
    insights_client.return_value.set_display_name.assert_not_called()
    get_scheduler.return_value.schedule.assert_called_once()


TEMP_TEST_REG_DIR = "/tmp/insights-client-registration"


@contextmanager
def _mock_no_register_files_machineid_present():
    # mock a directory with the fresh install files
    if not os.path.exists(TEMP_TEST_REG_DIR):
        os.mkdir(TEMP_TEST_REG_DIR)
    try:
        machine_id_path = os.path.join(TEMP_TEST_REG_DIR, "machine-id")
        with open(machine_id_path, "w") as machine_id_file:
            machine_id_file.write("id")
        yield
    finally:
        rmtree(TEMP_TEST_REG_DIR)


@patch('insights.client.utilities.constants.machine_id_file',
       TEMP_TEST_REG_DIR + '/machine-id')
@patch("insights.client.phase.v1.get_scheduler")
@patch("insights.client.phase.v1.InsightsClient")
@patch_insights_config
def test_post_update_register_machineid(insights_config, insights_client, get_scheduler):
    """
    Client run with --register.
        If machine-id found, exit with 0 exit code (don't kill parent)
        Also enable scheduling.
    """
    insights_config.return_value.load_all.return_value.register = True
    with _mock_no_register_files_machineid_present():
        with raises(SystemExit) as exc_info:
            post_update()
    assert exc_info.value.code == 0
    insights_client.return_value.clear_local_registration.assert_not_called()
    insights_client.return_value.set_display_name.assert_not_called()
    get_scheduler.return_value.schedule.assert_called_once()


@contextmanager
def _mock_no_machineid_present():
    # mock a directory with the fresh install files
    if not os.path.exists(TEMP_TEST_REG_DIR):
        os.mkdir(TEMP_TEST_REG_DIR)
    try:
        yield
    finally:
        rmtree(TEMP_TEST_REG_DIR)


@patch('insights.client.utilities.constants.machine_id_file',
       TEMP_TEST_REG_DIR + '/machine-id')
@patch("insights.client.phase.v1.get_scheduler")
@patch("insights.client.phase.v1.InsightsClient")
@patch_insights_config
def test_post_update_register_unregistered(insights_config, insights_client, get_scheduler):
    """
    Client run with --register.
        If unregistered, exit with 0 exit code (don't kill parent)
        Also enable scheduling.
    """
    insights_config.return_value.load_all.return_value.register = True
    with _mock_no_machineid_present():
        with raises(SystemExit) as exc_info:
            post_update()
    assert exc_info.value.code == 0
    insights_client.return_value.get_machine_id.assert_called_once()
    insights_client.return_value.clear_local_registration.assert_not_called()
    insights_client.return_value.set_display_name.assert_not_called()
    get_scheduler.return_value.schedule.assert_called_once()


@patch("insights.client.phase.v1.isfile", side_effect=[True])
@patch("insights.client.phase.v1.get_scheduler")
@patch("insights.client.phase.v1.InsightsClient")
@patch_insights_config
def test_post_update_unregister_registered(insights_config, insights_client, get_scheduler, _isfile):
    """
    Client run with --unregister.
        If registered, exit with 100 exit code
        Also disable scheduling.
    """
    insights_config.return_value.load_all.return_value.unregister = True
    insights_client.return_value.unregister = MagicMock(return_value=True)
    with raises(SystemExit) as exc_info:
        post_update()
    assert exc_info.value.code == 100
    insights_client.return_value.unregister.assert_called_once()
    insights_client.return_value.clear_local_registration.assert_not_called()
    insights_client.return_value.set_display_name.assert_not_called()
    get_scheduler.return_value.remove_scheduling.assert_called_once()


@patch("insights.client.phase.v1.isfile", side_effect=[False])
@patch("insights.client.phase.v1.get_scheduler")
@patch("insights.client.phase.v1.InsightsClient")
@patch_insights_config
def test_post_update_unregister_unregistered(insights_config, insights_client, get_scheduler, _isfile):
    """
    Client run with --unregister.
        If unregistered, exit with 101 exit code
    """
    insights_config.return_value.load_all.return_value.unregister = True
    insights_client.return_value.unregister = MagicMock(return_value=False)
    with raises(SystemExit) as exc_info:
        post_update()
    assert exc_info.value.code == 101
    insights_client.return_value.unregister.assert_called_once()
    insights_client.return_value.clear_local_registration.assert_not_called()
    insights_client.return_value.set_display_name.assert_not_called()
    get_scheduler.return_value.remove_scheduling.assert_not_called()


@patch("insights.client.phase.v1.isfile", side_effect=[False])
@patch("insights.client.phase.v1.get_scheduler")
@patch("insights.client.phase.v1.InsightsClient")
@patch_insights_config
def test_post_update_set_display_name_cli_no_register_unreg(insights_config, insights_client, get_scheduler, _isfile):
    """
    Client is unregistered, and run with --display-name but not --register
        Should exit with code 101 after registration check
    """
    insights_config.return_value.load_all.return_value.display_name = True
    insights_config.return_value.load_all.return_value._cli_opts = ['display_name']
    with raises(SystemExit) as exc_info:
        post_update()
    assert exc_info.value.code == 101
    insights_client.return_value.set_display_name.assert_not_called()
    get_scheduler.return_value.schedule.assert_not_called()


@patch("insights.client.phase.v1.isfile", side_effect=[True])
@patch("insights.client.phase.v1.get_scheduler")
@patch("insights.client.phase.v1.InsightsClient")
@patch_insights_config
def test_post_update_set_display_name_cli_no_register_reg(insights_config, insights_client, get_scheduler, _isfile):
    """
    Client is registered, and run with --display-name but not --register
        Should exit with code 100 after setting display name
    """
    insights_config.return_value.load_all.return_value.display_name = True
    insights_config.return_value.load_all.return_value._cli_opts = ['display_name']
    with raises(SystemExit) as exc_info:
        post_update()
    assert exc_info.value.code == 100
    insights_client.return_value.set_display_name.assert_called_once()
    get_scheduler.return_value.schedule.assert_not_called()


@patch("insights.client.phase.v1.isfile", side_effect=[False, False])
@patch("insights.client.phase.v1.get_scheduler")
@patch("insights.client.phase.v1.InsightsClient")
@patch_insights_config
def test_post_update_set_display_name_cli_register(insights_config, insights_client, get_scheduler, _isfile):
    """
    Client is and run with --display-name and --register
        Should set schedule and exit with code 0
        Display name is not explicitly set here
    """
    insights_config.return_value.load_all.return_value.register = True
    with raises(SystemExit) as exc_info:
        post_update()
    assert exc_info.value.code == 0
    insights_client.return_value.get_machine_id.assert_called_once()
    insights_client.return_value.set_display_name.assert_not_called()
    get_scheduler.return_value.schedule.assert_called_once()


@patch("insights.client.phase.v1.InsightsClient")
@patch_insights_config
def test_post_update_offline(insights_config, insights_client):
    """
    Offline mode short circuits this phase
    """
    insights_config.return_value.load_all.return_value.offline = True
    try:
        post_update()
    except SystemExit:
        pass
    insights_client.return_value.get_machine_id.assert_called_once()
    insights_client.return_value.clear_local_registration.assert_not_called()
    insights_client.return_value.set_display_name.assert_not_called()


@patch("insights.client.phase.v1.InsightsClient")
@patch_insights_config
def test_post_update_no_upload(insights_config, insights_client):
    """
    No-upload short circuits this phase
    """
    insights_config.return_value.load_all.return_value.no_upload = True
    try:
        post_update()
    except SystemExit:
        pass
    insights_client.return_value.get_machine_id.assert_called_once()
    insights_client.return_value.clear_local_registration.assert_not_called()
    insights_client.return_value.set_display_name.assert_not_called()


@patch("insights.client.phase.v1.isfile", side_effect=[True])
@patch("insights.client.phase.v1.InsightsClient")
@patch_insights_config
# @patch("insights.client.phase.v1.InsightsClient")
def test_exit_ok(insights_config, insights_client, _isfile):
    """
    Support collection replaces the normal client run.
    """
    with raises(SystemExit) as exc_info:
        post_update()
    assert exc_info.value.code == 0



import doctest
from datetime import datetime

from insights.parsers import ovirt_engine_log
from insights.tests import context_wrap


SERVER_LOG = """
2018-01-17 01:46:15,022+05 INFO  [org.jboss.as.server.deployment] (MSC service thread 1-1) WFLYSRV0027: Starting deployment of "restapi.war" (runtime-name: "restapi.war")
2018-01-17 01:46:15,022+05 INFO  [org.jboss.as.server.deployment] (MSC service thread 1-5) WFLYSRV0027: Starting deployment of "rhev.ear" (runtime-name: "rhev.ear")
2018-01-17 01:46:15,022+05 INFO  [org.jboss.as.server.deployment] (MSC service thread 1-6) WFLYSRV0027: Starting deployment of "apidoc.war" (runtime-name: "apidoc.war")
2018-01-17 01:46:15,022+05 INFO  [org.jboss.as.server.deployment] (MSC service thread 1-4) WFLYSRV0027: Starting deployment of "engine.ear" (runtime-name: "engine.ear")
2018-01-17 01:46:15,022+05 INFO  [org.jboss.as.server.deployment] (MSC service thread 1-3) WFLYSRV0027: Starting deployment of "ovirt-web-ui.war" (runtime-name: "ovirt-web-ui.war")
2018-01-17 01:46:15,035+05 INFO  [org.jboss.as.remoting] (MSC service thread 1-8) WFLYRMT0001: Listening on 127.0.0.1:8707
2018-01-17 01:46:15,064+05 INFO  [org.jboss.as.server.deployment] (MSC service thread 1-6) WFLYSRV0207: Starting subdeployment (runtime-name: "rhev.war")
2018-01-17 01:46:15,532+05 INFO  [org.wildfly.security] (MSC service thread 1-3) ELY00001: WildFly Elytron version 1.1.7.Final-redhat-1
2018-01-17 01:46:15,823+05 INFO  [org.wildfly.extension.undertow] (ServerService Thread Pool -- 47) WFLYUT0021: Registered web context: '/ovirt-engine/web-ui' for server 'default-server'
2018-01-17 01:46:15,823+05 INFO  [org.wildfly.extension.undertow] (ServerService Thread Pool -- 45) WFLYUT0021: Registered web context: '/ovirt-engine/rhev' for server 'default-server'
2018-01-17 01:46:15,823+05 INFO  [org.wildfly.extension.undertow] (ServerService Thread Pool -- 43) WFLYUT0021: Registered web context: '/ovirt-engine/apidoc' for server 'default-server'
2018-01-17 01:46:15,834+05 INFO  [org.jboss.as.server.deployment] (MSC service thread 1-5) WFLYSRV0207: Starting subdeployment (runtime-name: "bll.jar")
2018-01-17 01:46:15,834+05 INFO  [org.jboss.as.server.deployment] (MSC service thread 1-3) WFLYSRV0207: Starting subdeployment (runtime-name: "userportal.war")
2018-01-17 01:46:15,834+05 INFO  [org.jboss.as.server.deployment] (MSC service thread 1-5) WFLYSRV0207: Starting subdeployment (runtime-name: "enginesso.war")
2018-01-17 01:46:15,834+05 INFO  [org.jboss.as.server.deployment] (MSC service thread 1-4) WFLYSRV0207: Starting subdeployment (runtime-name: "welcome.war")
2018-01-17 01:46:15,834+05 INFO  [org.jboss.as.server.deployment] (MSC service thread 1-8) WFLYSRV0207: Starting subdeployment (runtime-name: "docs.war")
2018-01-17 01:46:15,834+05 INFO  [org.jboss.as.server.deployment] (MSC service thread 1-7) WFLYSRV0207: Starting subdeployment (runtime-name: "root.war")
2018-01-17 01:46:15,834+05 INFO  [org.jboss.as.server.deployment] (MSC service thread 1-6) WFLYSRV0207: Starting subdeployment (runtime-name: "services.war")
2018-01-17 01:46:16,834+05 INFO  [org.jboss.as.server.deployment] (MSC service thread 1-1) WFLYSRV0207: Starting subdeployment (runtime-name: "webadmin.war")
2018-01-17 01:46:17,739+05 WARN  [org.jboss.as.dependency.unsupported] (MSC service thread 1-7) WFLYSRV0019: Deployment "deployment.engine.ear" is using an unsupported module ("org.dom4j") which may be changed or removed in future versions without notice.
""".strip()

UI_LOG = """
2018-01-24 05:31:26,243+05 ERROR [org.ovirt.engine.ui.frontend.server.gwt.OvirtRemoteLoggingService] (default task-134) [] Permutation name: C068E8B2E40A504D3054A1BDCF2A72BB
2018-01-24 05:32:26,243+05 ERROR [org.ovirt.engine.ui.frontend.server.gwt.OvirtRemoteLoggingService] (default task-134) [] Uncaught exception: com.google.gwt.core.client.JavaScriptException: (TypeError)
""".strip()

# We cannot test this at the moment as LogFileOutput cannot read continuation multi line. Please see open issue#1256
CONSOLE_LOG = """
2018-08-01 09:15:14
Full thread dump OpenJDK 64-Bit Server VM (25.181-b13 mixed mode):

"ServerService Thread Pool -- 61" #118 prio=5 os_prio=0 tid=0x0000000007f1d000 nid=0x68c waiting on condition [0x00007f716d6bc000]
   java.lang.Thread.State: TIMED_WAITING (parking)
        at sun.misc.Unsafe.park(Native Method)
        - parking to wait for  <0x00000006c7fc5480> (a java.util.concurrent.locks.AbstractQueuedSynchronizer$ConditionObject)
        at java.util.concurrent.locks.LockSupport.parkNanos(LockSupport.java:215)
        at java.util.concurrent.locks.AbstractQueuedSynchronizer$ConditionObject.awaitNanos(AbstractQueuedSynchronizer.java:2078)
        at java.util.concurrent.ScheduledThreadPoolExecutor$DelayedWorkQueue.take(ScheduledThreadPoolExecutor.java:1093)
        at java.util.concurrent.ScheduledThreadPoolExecutor$DelayedWorkQueue.take(ScheduledThreadPoolExecutor.java:809)
        at java.util.concurrent.ThreadPoolExecutor.getTask(ThreadPoolExecutor.java:1074)
        at java.util.concurrent.ThreadPoolExecutor.runWorker(ThreadPoolExecutor.java:1134)
        at java.util.concurrent.ThreadPoolExecutor$Worker.run(ThreadPoolExecutor.java:624)
        at java.lang.Thread.run(Thread.java:748)
        at org.jboss.threads.JBossThread.run(JBossThread.java:320)

"ServerService Thread Pool -- 60" #117 prio=5 os_prio=0 tid=0x000000000604c800 nid=0x689 waiting on condition [0x00007f716d7bd000]
   java.lang.Thread.State: WAITING (parking)
        at sun.misc.Unsafe.park(Native Method)
        - parking to wait for  <0x00000006c7fc5480> (a java.util.concurrent.locks.AbstractQueuedSynchronizer$ConditionObject)
        at java.util.concurrent.locks.LockSupport.park(LockSupport.java:175)
        at java.util.concurrent.locks.AbstractQueuedSynchronizer$ConditionObject.await(AbstractQueuedSynchronizer.java:2039)
        at java.util.concurrent.ScheduledThreadPoolExecutor$DelayedWorkQueue.take(ScheduledThreadPoolExecutor.java:1088)
        at java.util.concurrent.ScheduledThreadPoolExecutor$DelayedWorkQueue.take(ScheduledThreadPoolExecutor.java:809)
        at java.util.concurrent.ThreadPoolExecutor.getTask(ThreadPoolExecutor.java:1074)
        at java.util.concurrent.ThreadPoolExecutor.runWorker(ThreadPoolExecutor.java:1134)
        at java.util.concurrent.ThreadPoolExecutor$Worker.run(ThreadPoolExecutor.java:624)
        at java.lang.Thread.run(Thread.java:748)
        at org.jboss.threads.JBossThread.run(JBossThread.java:320)

"xnio-file-watcher[Watcher for /usr/share/ovirt-engine/branding/rhv-2.brand/applications/rhv.ear/rhv.war/]-0" #98 daemon prio=5 os_prio=0 tid=0x00000000064a9000 nid=0x66d waiting on condition [0x00007f716e4a2000]
   java.lang.Thread.State: WAITING (parking)
        at sun.misc.Unsafe.park(Native Method)
        - parking to wait for  <0x00000006c8bf3c28> (a java.util.concurrent.locks.AbstractQueuedSynchronizer$ConditionObject)
        at java.util.concurrent.locks.LockSupport.park(LockSupport.java:175)
        at java.util.concurrent.locks.AbstractQueuedSynchronizer$ConditionObject.await(AbstractQueuedSynchronizer.java:2039)
        at java.util.concurrent.LinkedBlockingDeque.takeFirst(LinkedBlockingDeque.java:492)
        at java.util.concurrent.LinkedBlockingDeque.take(LinkedBlockingDeque.java:680)
        at sun.nio.fs.AbstractWatchService.take(AbstractWatchService.java:118)
        at org.xnio.nio.WatchServiceFileSystemWatcher.run(WatchServiceFileSystemWatcher.java:86)
        at java.lang.Thread.run(Thread.java:748)

"Thread-61" #97 daemon prio=5 os_prio=0 tid=0x00000000064a8800 nid=0x66b runnable [0x00007f716e5a3000]
   java.lang.Thread.State: RUNNABLE
        at sun.nio.fs.LinuxWatchService.poll(Native Method)
        at sun.nio.fs.LinuxWatchService.access$600(LinuxWatchService.java:47)
        at sun.nio.fs.LinuxWatchService$Poller.run(LinuxWatchService.java:314)
        at java.lang.Thread.run(Thread.java:748)

"xnio-file-watcher[Watcher for /usr/share/ovirt-web-ui/ovirt-web-ui.war/]-0" #96 daemon prio=5 os_prio=0 tid=0x000000000639c800 nid=0x66a waiting on condition [0x00007f716e6a4000]
   java.lang.Thread.State: WAITING (parking)
        at sun.misc.Unsafe.park(Native Method)
        - parking to wait for  <0x00000006c8bc5850> (a java.util.concurrent.locks.AbstractQueuedSynchronizer$ConditionObject)
        at java.util.concurrent.locks.LockSupport.park(LockSupport.java:175)
        at java.util.concurrent.locks.AbstractQueuedSynchronizer$ConditionObject.await(AbstractQueuedSynchronizer.java:2039)
        at java.util.concurrent.LinkedBlockingDeque.takeFirst(LinkedBlockingDeque.java:492)
        at java.util.concurrent.LinkedBlockingDeque.take(LinkedBlockingDeque.java:680)
        at sun.nio.fs.AbstractWatchService.take(AbstractWatchService.java:118)
        at org.xnio.nio.WatchServiceFileSystemWatcher.run(WatchServiceFileSystemWatcher.java:86)
        at java.lang.Thread.run(Thread.java:748)

"Thread-62" #95 daemon prio=5 os_prio=0 tid=0x0000000005933800 nid=0x669 runnable [0x00007f716e7a5000]
   java.lang.Thread.State: RUNNABLE
        at sun.nio.fs.LinuxWatchService.poll(Native Method)
        at sun.nio.fs.LinuxWatchService.access$600(LinuxWatchService.java:47)
        at sun.nio.fs.LinuxWatchService$Poller.run(LinuxWatchService.java:314)
        at java.lang.Thread.run(Thread.java:748)
""".strip()

ENGINE_LOG = """
2018-08-06 04:06:33,229+05 INFO  [org.ovirt.engine.core.bll.utils.ThreadPoolMonitoringService] (EE-ManagedThreadFactory-engineThreadMonitoring-Thread-1) [] Thread pool 'engine' is using 0 threads out of 500, 8 threads waiting for tasks and 0 tasks in queue.
2018-08-06 04:06:33,229+05 INFO  [org.ovirt.engine.core.bll.utils.ThreadPoolMonitoringService] (EE-ManagedThreadFactory-engineThreadMonitoring-Thread-1) [] Thread pool 'engineScheduled' is using 0 threads out of 100, 100 threads waiting for tasks.
2018-08-06 04:06:33,229+05 INFO  [org.ovirt.engine.core.bll.utils.ThreadPoolMonitoringService] (EE-ManagedThreadFactory-engineThreadMonitoring-Thread-1) [] Thread pool 'engineThreadMonitoring' is using 1 threads out of 1, 0 threads waiting for tasks.
2018-08-06 04:06:33,229+05 INFO  [org.ovirt.engine.core.bll.utils.ThreadPoolMonitoringService] (EE-ManagedThreadFactory-engineThreadMonitoring-Thread-1) [] Thread pool 'hostUpdatesChecker' is using 0 threads out of 5, 5 threads waiting for tasks.
2018-08-06 04:16:33,231+05 INFO  [org.ovirt.engine.core.bll.utils.ThreadPoolMonitoringService] (EE-ManagedThreadFactory-engineThreadMonitoring-Thread-1) [] Thread pool 'commandCoordinator' is using 0 threads out of 10, 2 threads waiting for tasks.
2018-08-06 04:16:33,231+05 INFO  [org.ovirt.engine.core.bll.utils.ThreadPoolMonitoringService] (EE-ManagedThreadFactory-engineThreadMonitoring-Thread-1) [] Thread pool 'default' is using 0 threads out of 1, 5 threads waiting for tasks.
2018-08-06 04:16:33,231+05 INFO  [org.ovirt.engine.core.bll.utils.ThreadPoolMonitoringService] (EE-ManagedThreadFactory-engineThreadMonitoring-Thread-1) [] Thread pool 'engine' is using 0 threads out of 500, 8 threads waiting for tasks and 0 tasks in queue.
2018-08-06 04:16:33,231+05 INFO  [org.ovirt.engine.core.bll.utils.ThreadPoolMonitoringService] (EE-ManagedThreadFactory-engineThreadMonitoring-Thread-1) [] Thread pool 'engineScheduled' is using 0 threads out of 100, 100 threads waiting for tasks.
2018-08-06 04:16:33,231+05 INFO  [org.ovirt.engine.core.bll.utils.ThreadPoolMonitoringService] (EE-ManagedThreadFactory-engineThreadMonitoring-Thread-1) [] Thread pool 'engineThreadMonitoring' is using 1 threads out of 1, 0 threads waiting for tasks.
2018-08-06 04:16:33,231+05 INFO  [org.ovirt.engine.core.bll.utils.ThreadPoolMonitoringService] (EE-ManagedThreadFactory-engineThreadMonitoring-Thread-1) [] Thread pool 'hostUpdatesChecker' is using 0 threads out of 5, 5 threads waiting for tasks.
2018-08-06 04:26:33,233+05 INFO  [org.ovirt.engine.core.bll.utils.ThreadPoolMonitoringService] (EE-ManagedThreadFactory-engineThreadMonitoring-Thread-1) [] Thread pool 'commandCoordinator' is using 0 threads out of 10, 2 threads waiting for tasks.
2018-08-06 04:26:33,233+05 INFO  [org.ovirt.engine.core.bll.utils.ThreadPoolMonitoringService] (EE-ManagedThreadFactory-engineThreadMonitoring-Thread-1) [] Thread pool 'default' is using 0 threads out of 1, 5 threads waiting for tasks.
2018-08-06 04:26:33,233+05 INFO  [org.ovirt.engine.core.bll.utils.ThreadPoolMonitoringService] (EE-ManagedThreadFactory-engineThreadMonitoring-Thread-1) [] Thread pool 'engine' is using 0 threads out of 500, 8 threads waiting for tasks and 0 tasks in queue.
2018-08-22 00:16:14,357+05 INFO  [org.ovirt.engine.core.vdsbroker.vdsbroker.HotUnplugLeaseVDSCommand] (default task-133) [e3bc976c-bc3e-4b41-807f-3a518169ad18] START, HotUnplugLeaseVDSCommand(HostName = example.com, LeaseVDSParameters:{hostId='bfa308ab-5add-4ad7-8f1c-389cb8dcf703', vmId='789489a3-be62-40e4-b13e-beb34ba5ff93'}), log id: 7a634963
""".strip()

BOOT_LOG = """
03:46:17,790 INFO  [org.jboss.modules] JBoss Modules version 1.6.4.Final-redhat-1
03:46:18,067 INFO  [org.jboss.msc] JBoss MSC version 1.2.7.SP1-redhat-1
03:46:18,181 INFO  [org.jboss.as] WFLYSRV0049: JBoss EAP 7.1.3.GA (WildFly Core 3.0.16.Final-redhat-1) starting
03:46:19,126 INFO  [org.jboss.as.controller.management-deprecated] WFLYCTL0028: Attribute 'security-realm' in the resource at address '/core-service=management/management-interface=native-interface' is deprecated, and may be removed in future version. See the attribute description in the output of the read-resource-description operation to learn more about the deprecation.
03:46:19,128 INFO  [org.jboss.as.controller.management-deprecated] WFLYCTL0028: Attribute 'security-realm' in the resource at address '/core-service=management/management-interface=http-interface' is deprecated, and may be removed in future version. See the attribute description in the output of the read-resource-description operation to learn more about the deprecation.
03:46:19,208 INFO  [org.jboss.as.server.deployment.scanner] WFLYDS0004: Found restapi.war in deployment directory. To trigger deployment create a file called restapi.war.dodeploy
03:46:19,208 INFO  [org.jboss.as.server.deployment.scanner] WFLYDS0004: Found engine.ear in deployment directory. To trigger deployment create a file called engine.ear.dodeploy
03:46:19,208 INFO  [org.jboss.as.server.deployment.scanner] WFLYDS0004: Found ovirt-web-ui.war in deployment directory. To trigger deployment create a file called ovirt-web-ui.war.dodeploy
03:46:19,208 INFO  [org.jboss.as.server.deployment.scanner] WFLYDS0004: Found apidoc.war in deployment directory. To trigger deployment create a file called apidoc.war.dodeploy
03:46:19,208 INFO  [org.jboss.as.server.deployment.scanner] WFLYDS0004: Found rhv.ear in deployment directory. To trigger deployment create a file called rhv.ear.dodeploy
03:46:19,238 INFO  [org.jboss.as.server] WFLYSRV0039: Creating http management service using socket-binding (management)
03:46:19,242 INFO  [org.xnio] XNIO version 3.5.5.Final-redhat-1
03:46:19,250 INFO  [org.xnio.nio] XNIO NIO Implementation Version 3.5.5.Final-redhat-1
""".strip()


def test_server_log():
    server_log = ovirt_engine_log.ServerLog(context_wrap(SERVER_LOG))
    assert 'is using an unsupported module' in server_log
    assert len(list(server_log.get_after(datetime(2018, 1, 17, 1, 46, 16, 0)))) == 2

    matched_line = '2018-01-17 01:46:17,739+05 WARN  [org.jboss.as.dependency.unsupported] (MSC service thread 1-7) WFLYSRV0019: Deployment "deployment.engine.ear" is using an unsupported module ("org.dom4j") which may be changed or removed in future versions without notice.'
    assert server_log.get('WARN')[-1].get('raw_message') == matched_line

    sec_lines = server_log.get('org.wildfly.security')
    assert len(sec_lines) == 1
    assert sec_lines[0]['level'] == 'INFO'


def test_ui_log():
    ui_log = ovirt_engine_log.UILog(context_wrap(UI_LOG))
    assert 'Permutation name' in ui_log
    assert len(list(ui_log.get_after(datetime(2018, 1, 24, 5, 31, 26, 0)))) == 2

    exception_lines = ui_log.get('Uncaught exception')
    assert len(exception_lines) == 1
    assert exception_lines[0].get('procname') == 'org.ovirt.engine.ui.frontend.server.gwt.OvirtRemoteLoggingService'
    assert exception_lines[0].get('level') == 'ERROR'


def test_engine_log():
    engine_log = ovirt_engine_log.EngineLog(context_wrap(ENGINE_LOG))
    assert "Thread pool 'engine'" in engine_log
    assert len(list(engine_log.get_after(datetime(2018, 8, 6, 4, 16, 33, 0)))) == 10

    matched_line = "2018-08-06 04:16:33,231+05 INFO  [org.ovirt.engine.core.bll.utils.ThreadPoolMonitoringService] (EE-ManagedThreadFactory-engineThreadMonitoring-Thread-1) [] Thread pool 'hostUpdatesChecker' is using 0 threads out of 5, 5 threads waiting for tasks."
    assert engine_log.get('hostUpdatesChecker')[-1].get('raw_message') == matched_line
    assert engine_log.get('vdsbroker')[-1].get('procname') == 'org.ovirt.engine.core.vdsbroker.vdsbroker.HotUnplugLeaseVDSCommand'


def test_boot_log():
    boot_log = ovirt_engine_log.BootLog(context_wrap(BOOT_LOG))
    assert "Creating http management service using socket-binding" in boot_log
    xnio_lines = boot_log.get('xnio.nio')
    assert len(xnio_lines) == 1
    assert xnio_lines[0].get('procname') == 'org.xnio.nio'
    assert xnio_lines[0].get('level') == 'INFO'
    assert xnio_lines[0].get('message') == 'XNIO NIO Implementation Version 3.5.5.Final-redhat-1'

    log_line = '2018-01-17 INFO  [org.jboss.as.server.deployment] (MSC service thread 1-1) WFLYSRV0027: Starting deployment of "restapi.war" (runtime-name: "restapi.war")'
    boot_log = ovirt_engine_log.BootLog(context_wrap(log_line))
    assert "restapi" in boot_log
    assert boot_log.get('restapi')[0]['raw_message'] == log_line


def test_documentation():
    failed_count, tests = doctest.testmod(
        ovirt_engine_log,
        globs={'server_log': ovirt_engine_log.ServerLog(context_wrap(SERVER_LOG)),
               'boot_log': ovirt_engine_log.BootLog(context_wrap(BOOT_LOG)),
               'engine_log': ovirt_engine_log.EngineLog(context_wrap(ENGINE_LOG)),
               'ui_log': ovirt_engine_log.UILog(context_wrap(UI_LOG))}
    )
    assert failed_count == 0



import pytest
from insights.client.config import InsightsConfig
from insights.client.connection import InsightsConnection
from mock.mock import patch


class MockSession(object):
    def __init__(self):
        self.status_code = None
        self.text = None
        self.content = '{"display_name": "test"}'

    def get(self, url=None, timeout=None, headers=None, data=None):
        return MockResponse(self.status_code, self.text, self.content)

    def put(self, url=None, timeout=None, headers=None, data=None):
        return MockResponse(self.status_code, self.text, None)


class MockResponse(object):
    def __init__(self, expected_status, expected_text, expected_content):
        self.status_code = expected_status
        self.text = expected_text
        self.content = expected_content


def mock_init_session(obj):
    return MockSession()


def mock_get_proxies(obj):
    return


@pytest.mark.skip(reason='No time to fix this for double-API calling')
@patch('insights.client.connection.InsightsConnection._init_session',
       mock_init_session)
@patch('insights.client.connection.InsightsConnection.get_proxies',
       mock_get_proxies)
@patch('insights.client.utilities.constants.machine_id_file',
       '/tmp/machine-id')
def test_set_ansible_host():
    conf = InsightsConfig()
    c = InsightsConnection(conf)
    c.session.status_code = 200
    assert c.set_ansible_host('GO STICK YOUR HEAD IN A PIG')
    c.session.status_code = 404
    assert not c.set_ansible_host('GO STICK YOUR HEAD IN A PIG')
    c.session.status_code = 500
    c.session.text = 'oops'
    assert not c.set_ansible_host('GO STICK YOUR HEAD IN A PIG')


def test_ansible_host_no_reg_forces_legacy_false():
    '''
    When not specifying --register, using --ansible-host on the CLI forces legacy_upload to False
    '''
    conf = InsightsConfig(register=False, ansible_host="test", legacy_upload=True)
    conf._cli_opts = ["ansible_host"]
    conf._imply_options()
    assert not conf.legacy_upload
    conf = InsightsConfig(register=False, ansible_host="test", legacy_upload=False)
    conf._cli_opts = ["ansible_host"]
    conf._imply_options()
    assert not conf.legacy_upload


def test_ansible_host_reg_legacy_no_change():
    '''
    When specifying --register, using --ansible-host on the CLI does not affect legacy_upload
    '''
    conf = InsightsConfig(register=True, ansible_host="test", legacy_upload=True)
    conf._cli_opts = ["ansible_host"]
    conf._imply_options()
    assert conf.legacy_upload
    conf = InsightsConfig(register=True, ansible_host="test", legacy_upload=False)
    conf._cli_opts = ["ansible_host"]
    conf._imply_options()
    assert not conf.legacy_upload



from mock.mock import patch

from insights.cleaner import Cleaner
from insights.client.config import InsightsConfig

hostname = 'test1.abc.com'
line = "test1.abc.com, 10.0.0.1 test1.abc.loc 20.1.4.7 smtp.abc.com 10.1.2.7 lite.abc.com"


@patch("insights.cleaner.MAX_LINE_LENGTH", len(hostname) + 1)
@patch("insights.cleaner.logger")
def test_max_line_length_less(logger):
    c = InsightsConfig(obfuscate=True, obfuscate_hostname=True, hostname=hostname)
    pp = Cleaner(c, {}, hostname)
    result = pp.clean_content(line)
    logger.debug.assert_called_once_with('Extra-long line is truncated ...')
    assert 'example.com' in result
    assert '10.230.230' not in result
    assert result[-1] == ','


@patch("insights.cleaner.MAX_LINE_LENGTH", len(line) + 10)
@patch("insights.cleaner.logger")
def test_max_line_length_good(logger):
    c = InsightsConfig(obfuscate=True, obfuscate_hostname=True, hostname=hostname)
    pp = Cleaner(c, {}, hostname)
    result = pp.clean_content(line)
    logger.debug.assert_not_called()
    assert 'example.com' in result
    assert '10.230.230' in result
    assert result.endswith('example.com')



"""
Cgroups - File ``/proc/cgroups``
================================

This parser reads the content of ``/proc/cgroups``.
This file shows the control groups information of system.

Sample ``/proc/cgroups`` file::

    #subsys_name	hierarchy	num_cgroups	enabled
    cpuset	10	48	1
    cpu	2	232	1
    cpuacct	2	232	1
    memory	5	232	1
    devices	6	232	1
    freezer	3	48	1
    net_cls	4	48	1
    blkio	9	232	1
    perf_event	8	48	1
    hugetlb	11	48	1
    pids	7	232	1
    net_prio	4	48	1


Examples:

    >>> i_cgroups.get_num_cgroups("memory")
    232
    >>> i_cgroups.is_subsys_enabled("memory")
    True
    >>> i_cgroups.data[0].get('hierarchy')
    '10'
    >>> i_cgroups.subsystems["memory"]["enabled"]
    '1'

"""

from .. import parser, Parser
from insights.specs import Specs
from . import parse_delimited_table


@parser(Specs.cgroups)
class Cgroups(Parser):
    """
    Class ``Cgroups`` parses the content of the ``/proc/cgroups``.

    Attributes:
        data (list): A list of the subsystem cgroup information
        subsystems (dict): A dict of all subsystems, key is subsystem name and value is dict with keys: hierarchy, num_cgroups, enabled
    """

    def parse_content(self, content):
        # replace #subsys_name to subsys_name
        self.data = parse_delimited_table(content,
                                          heading_ignore=['#subsys_name', 'hierarchy'],
                                          header_substitute=[('#subsys_name', 'subsys_name')])
        self.subsystems = {}
        for subsys in self.data:
            self.subsystems[subsys.pop("subsys_name")] = subsys

    def get_num_cgroups(self, i_subsys_name):
        """
        Get value of cgroup number for specified subsystem, raise exception if keyword not found.

        Example:

            >>> i_cgroups.get_num_cgroups("memory")
            232
            >>> i_cgroups.get_num_cgroups('hugetlb')
            48

        Args:

            i_subsys_name (str): specified subsystem name.

        Returns:

            value (int): Int value of the specified subsystem cgroups

        Raises:
            KeyError: Exception is raised if given subsystem name is wrong

        """
        if i_subsys_name in self.subsystems:
            return int(self.subsystems[i_subsys_name]["num_cgroups"])
        # raise keyerror exception if give the wrong subsys_name
        raise KeyError("Gave wrong subsys_name: {0}".format(i_subsys_name))

    def is_subsys_enabled(self, i_subsys_name):
        """
        Get enable or not of cgroup of specified subsystem, raise exception if keyword not found.

        Example:

            >>> i_cgroups.is_subsys_enabled("memory")
            True
            >>> i_cgroups.is_subsys_enabled('hugetlb')
            True

        Args:

            i_subsys_name (str): specified subsystem name.

        Returns:

            value (boolean): Return True if the cgroup of specified subsystem is enabled, else return False

        Raises:
            KeyError: Exception is raised if given subsystem name is wrong

        """
        if i_subsys_name in self.subsystems:
            return self.subsystems[i_subsys_name]["enabled"] == "1"
        # raise keyerror exception if give the wrong subsys_name
        raise KeyError("Gave wrong subsys_name: {0}".format(i_subsys_name))



import os

from pytest import mark

from insights.client.archive import InsightsArchive
from insights.client.config import InsightsConfig
from insights.cleaner import Cleaner

test_file_data = 'test\nabcd\n1234\npwd: p4ssw0rd\n'


@mark.parametrize(
    ("line", "expected"),
    [
        (
            "what's your name? what day is today?",
            "what's your keyword0? what keyword1 is tokeyword1?",
        ),
    ],
)
@mark.parametrize("obfuscate", [True, False])
def test_clean_file_patterns_exclude_regex(obfuscate, line, expected):
    conf = InsightsConfig(obfuscate=obfuscate)
    arch = InsightsArchive(conf)
    arch.create_archive_dir()

    # put something in the archive to redact
    test_file = os.path.join(arch.archive_dir, 'test.file')
    with open(test_file, 'w') as t:
        t.write(test_file_data)

    rm_conf = {'patterns': {'regex': ['12.*4', '^abcd']}}
    pp = Cleaner(conf, rm_conf)
    pp.clean_file(test_file, [])
    with open(test_file, 'r') as t:
        data = [i.strip() for i in t.readlines()]
        assert '1234' not in data
        assert 'abcd' not in data
    arch.delete_archive_dir()


@mark.parametrize("obfuscate", [True, False])
def test_clean_file_patterns_empty_result(obfuscate):
    conf = InsightsConfig(obfuscate=obfuscate)
    arch = InsightsArchive(conf)
    arch.create_archive_dir()

    # put something in the archive to redact
    test_file = os.path.join(arch.archive_dir, 'test.file')
    with open(test_file, 'w') as t:
        t.write(test_file_data)

    rm_conf = {'patterns': {'regex': ['test', 'pwd', '12.*4', '^abcd']}}
    pp = Cleaner(conf, rm_conf)
    pp.clean_file(test_file)
    # file is cleaned to empty, hence it was removed
    assert not os.path.exists(test_file)
    arch.delete_archive_dir()


@mark.parametrize("obfuscate", [True, False])
def test_clean_file_patterns_exclude_no_regex(obfuscate):
    conf = InsightsConfig(obfuscate=obfuscate)
    arch = InsightsArchive(conf)
    arch.create_archive_dir()

    # put something in the archive to redact
    test_file = os.path.join(arch.archive_dir, 'test.file')
    with open(test_file, 'w') as t:
        t.write(test_file_data)

    rm_conf = {'patterns': ['1234', 'abcd']}
    pp = Cleaner(conf, rm_conf)
    pp.clean_file(test_file, [])
    with open(test_file, 'r') as t:
        data = [i.strip() for i in t.readlines()]
        assert '1234' not in data
        assert 'abcd' not in data
    arch.delete_archive_dir()


@mark.parametrize("obfuscate", [True, False])
def test_clean_file_patterns_exclude_empty(obfuscate):
    conf = InsightsConfig(obfuscate=obfuscate)
    arch = InsightsArchive(conf)
    arch.create_archive_dir()

    # put something in the archive to redact
    test_file = os.path.join(arch.archive_dir, 'test.file')
    with open(test_file, 'w') as t:
        t.write(test_file_data)

    rm_conf = {'patterns': []}
    pp = Cleaner(conf, rm_conf)
    pp.clean_file(test_file, [])
    # file is not changed
    with open(test_file, 'r') as t:
        assert test_file_data == ''.join(t.readlines())
    arch.delete_archive_dir()


@mark.parametrize("obfuscate", [True, False])
def test_clean_file_patterns_exclude_none(obfuscate):
    conf = InsightsConfig(obfuscate=obfuscate)
    arch = InsightsArchive(conf)
    arch.create_archive_dir()

    # put something in the archive to redact
    test_file = os.path.join(arch.archive_dir, 'test.file')
    with open(test_file, 'w') as t:
        t.write(test_file_data)

    pp = Cleaner(conf, None)
    pp.clean_file(test_file, [])
    # file is not changed
    with open(test_file, 'r') as t:
        assert test_file_data == ''.join(t.readlines())
    arch.delete_archive_dir()



import doctest
import pytest

from insights.core.exceptions import ParseException
from insights.parsers import cups_confs
from insights.parsers.cups_confs import CupsdConf, CupsFilesConf, CupsBrowsedConf
from insights.parsr.query import first, last
from insights.tests import context_wrap

CUPSD_CONF = """
MaxLogSize 0
LogLevel warn

# Only listen for connections from the local machine.
Listen localhost:631
Listen /var/run/cups/cups.sock
Browsing On
BrowseLocalProtocols dnssd
DefaultAuthType Basic
WebInterface Yes
IdleExitTimeout 0

<Location />
  Order allow,deny
</Location>

<Location /admin>
  Order allow,deny
</Location>

<Location /admin/conf>
  AuthType Default
  Require user @SYSTEM
  Order allow,deny
</Location>

<Location /admin/log>
  AuthType Default
  Require user @SYSTEM
  Order allow,deny
</Location>

<Policy default>
  JobPrivateAccess default
  JobPrivateValues default
  SubscriptionPrivateAccess default
  SubscriptionPrivateValues default

  <Limit Create-Job Print-Job Print-URI Validate-Job>
    Order deny,allow
  </Limit>

  <Limit Pause-Printer Resume-Printer Enable-Printer Disable-Printer Pause-Printer-After-Current-Job Hold-New-Jobs Release-Held-New-Jobs Deactivate-Printer Activate-Printer Restart-Printer Shutdown-Printer Startup-Printer Promote-Job Schedule-Job-After Cancel-Jobs CUPS-Accept-Jobs CUPS-Reject-Jobs>
    AuthType Default
    Require user @SYSTEM
    Order deny,allow
  </Limit>

  <Limit Cancel-Job CUPS-Authenticate-Job>
    Require user @OWNER @SYSTEM
    Order deny,allow
  </Limit>

  <Limit All>
    Order deny,allow
  </Limit>
</Policy>
""".strip()

CUPSD_CONF_EXAMPLE = """
MaxLogSize 0
LogLevel warn
Listen localhost:631
Listen /var/run/cups/cups.sock
Browsing On
BrowseLocalProtocols dnssd

<Location />
Order allow,deny
</Location>

<Policy default>
JobPrivateAccess default
JobPrivateValues default
SubscriptionPrivateAccess default
SubscriptionPrivateValues default

<Limit Send-Document Send-URI Hold-Job Release-Job Restart-Job Purge-Jobs Set-Job-Attributes Create-Job-Subscription Renew-Subscription Cancel-Subscription Get-Notifications Reprocess-Job Cancel-Current-Job Suspend-Current-Job Resume-Job Cancel-My-Jobs Close-Job CUPS-Move-Job CUPS-Get-Document>
Require user @OWNER @SYSTEM
Order deny,allow
</Limit>

<Limit All>
Order deny,allow
</Limit>
</Policy>
""".strip()

CUPSD_CONF_ERROR = """
</Limit>

<Limit All>
Order deny,allow
</Limit>
</Policy>
""".strip()

CUPS_FILES_CONF = """
FatalErrors config
SyncOnClose Yes
# Default user and group for filters/backends/helper programs; this cannot be
# any user or group that resolves to ID 0 for security reasons...
#User lp
#Group lp
# Administrator user group, used to match @SYSTEM in cupsd.conf policy rules...
# This cannot contain the Group value for security reasons...
SystemGroup sys root
SystemGroup wheel
# User that is substituted for unauthenticated (remote) root accesses...
#RemoteRoot remroot
AccessLog syslog
ErrorLog syslog
PageLog syslog
""".strip()

CUPS_FILES_CONF_EMPTY = """
""".strip()


CUPS_BROWSED_CONF = """
# Which protocols will we use to discover printers on the network?
# Can use DNSSD and/or CUPS and/or LDAP, or 'none' for neither.

BrowseRemoteProtocols dnssd cups
BrowseRemoteProtocols none
BrowseAllow 192.168.0.1
BrowseAllow 192.168.0.255
BrowseAllow cups.example.com
BrowseAllow 192.168.0.255
""".strip()

CUPS_BROWSED_CONF_CONF_EMPTY = """
""".strip()


def test_cupsd_conf():
    context = context_wrap(CUPSD_CONF)
    result = CupsdConf(context)
    assert result['Listen'][first].value == 'localhost:631'
    assert result['Listen'][last].value == '/var/run/cups/cups.sock'
    assert result[('Policy', 'default')][('Limit', 'Cancel-Job')]['Require'][first].value == "user @OWNER @SYSTEM"
    assert result[('Policy', 'default')][('Limit', 'CUPS-Authenticate-Job')]['Require'][first].value == "user @OWNER @SYSTEM"
    assert result[('Location', '/admin')]['Order'][last].value == 'allow,deny'


def test_cupsd_conf_error():
    with pytest.raises(ParseException) as exc:
        CupsdConf(context_wrap(CUPSD_CONF_ERROR))
    assert str(exc.value) == "There was an exception when parsing the cupsd.conf file"


def test_cups_files_conf():
    result = CupsFilesConf(context_wrap(CUPS_FILES_CONF))
    assert len(result) == 6
    assert 'SyncOnClose' in result
    assert result['SystemGroup'] == ['sys', 'root', 'wheel']
    assert result['AccessLog'] == 'syslog'


def test_cups_files_conf_empty():
    with pytest.raises(ParseException) as exc:
        CupsFilesConf(context_wrap(CUPS_FILES_CONF_EMPTY))
    assert str(exc.value) == "Empty Content"


def test_cups_browsed_files_conf():
    result = CupsBrowsedConf(context_wrap(CUPS_BROWSED_CONF))
    assert len(result) == 2
    assert 'BrowseRemoteProtocols' in result
    assert result['BrowseRemoteProtocols'] == ['dnssd cups', 'none']
    assert result['BrowseAllow'] == ['192.168.0.1', '192.168.0.255', 'cups.example.com', '192.168.0.255']


def test_cups_browsed_conf_empty():
    with pytest.raises(ParseException) as exc:
        CupsBrowsedConf(context_wrap(CUPS_BROWSED_CONF_CONF_EMPTY))
    assert str(exc.value) == "Empty Content"


def test_doc():
    env = {
        'cupsd_conf': CupsdConf(context_wrap(CUPSD_CONF_EXAMPLE)),
        'cups_files_conf': CupsFilesConf(context_wrap(CUPS_FILES_CONF)),
        'cups_browsed_conf': CupsBrowsedConf(context_wrap(CUPS_BROWSED_CONF))
    }
    failed, total = doctest.testmod(cups_confs, globs=env)
    assert failed == 0



"""
VirtlogdConf - file ``/etc/libvirt/virtlogd.conf``
==================================================

The VirtlogdConf class parses the file ``/etc/libvirt/virtlogd.conf``.
"""
from .. import LegacyItemAccess, Parser, parser
from insights.parsers import split_kv_pairs, get_active_lines
from insights.specs import Specs


@parser(Specs.virtlogd_conf)
class VirtlogdConf(LegacyItemAccess, Parser):
    """Parse content of ``/etc/libvirt/virtlogd.conf``. The virtlogd.conf
    is in the standard ``conf`` file format and is read by the base parser
    class ``LegacyItemAccess``.

    Sample ``/etc/libvirt/virtlogd.conf`` file::

        # Master virtlogd daemon configuration file
        #

        #################################################################
        #
        # Logging controls
        #

        # Logging level: 4 errors, 3 warnings, 2 information, 1 debug
        # basically 1 will log everything possible
        #log_level = 3

        # Logging filters:
        # A filter allows to select a different logging level for a given category
        # of logs
        # The format for a filter is one of:
        #    x:name
        #    x:+name
        #      where name is a string which is matched against source file name,
        #      e.g., "remote", "qemu", or "util/json", the optional "+" prefix
        #      tells libvirt to log stack trace for each message matching name,
        #      and x is the minimal level where matching messages should be logged:
        #    1: DEBUG
        #    2: INFO
        #    3: WARNING
        #    4: ERROR
        #
        # Multiple filter can be defined in a single @filters, they just need to be
        # separated by spaces.
        #
        # e.g. to only get warning or errors from the remote layer and only errors
        # from the event layer:
        #log_filters="3:remote 4:event"

        # Logging outputs:
        # An output is one of the places to save logging information
        # The format for an output can be:
        #    x:stderr
        #      output goes to stderr
        #    x:syslog:name
        #      use syslog for the output and use the given name as the ident
        #    x:file:file_path
        #      output to a file, with the given filepath
        #    x:journald
        #      ouput to the systemd journal
        # In all case the x prefix is the minimal level, acting as a filter
        #    1: DEBUG
        #    2: INFO
        #    3: WARNING
        #    4: ERROR
        #
        # Multiple output can be defined, they just need to be separated by spaces.
        # e.g. to log all warnings and errors to syslog under the virtlogd ident:
        #log_outputs="3:syslog:virtlogd"
        #

        # The maximum number of concurrent client connections to allow
        # over all sockets combined.
        #max_clients = 1024


        # Maximum file size before rolling over. Defaults to 2 MB
        #max_size = 2097152

        # Maximum number of backup files to keep. Defaults to 3,
        # not including the primary active file
        max_backups = 3


    Examples:
        >>> conf.get('max_backups')
        '3'

    Attributes:
        data (dict): Ex: ``{'max_backups': '3'}``
    """
    def parse_content(self, content):
        self.data = split_kv_pairs(get_active_lines(content))



"""
Grubby
======
Combiner for command ``/usr/sbin/grubby`` parsers.

This combiner uses the parsers:
:class:`insights.parsers.grubby.GrubbyDefaultIndex`,
:class:`insights.parsers.grubby.GrubbyInfoAll`.
"""

from insights.core.exceptions import ParseException
from insights.core.plugins import combiner
from insights.parsers.grubby import GrubbyDefaultIndex, GrubbyInfoAll


@combiner(GrubbyInfoAll, GrubbyDefaultIndex)
class Grubby(object):
    """
    Combine command "grubby" parsers into one Combiner.

    Attributes:
        boot_entries (dict): All boot entries indexed by the entry "index"
        default_index (int): The numeric index of the default boot entry
        default_boot_entry (dict): The boot information for default kernel
        default_kernel (str): The path of the default kernel

    Raises:
        ParseException: when parsing into error.
    """

    def __init__(self, grubby_info_all, grubby_default_index):
        self.boot_entries = grubby_info_all.boot_entries
        self.default_index = grubby_default_index.default_index

        if self.default_index not in self.boot_entries:
            raise ParseException(
                "DEFAULT index %s not exist in parsed boot_entries: %s"
                % (self.default_index, list(self.boot_entries.keys()))
            )
        self.default_boot_entry = self.boot_entries[self.default_index]

        self.default_kernel = self.default_boot_entry.get("kernel")
        if not self.default_kernel:
            raise ParseException(
                "DEFAULT kernel-path not exist in default-index: %s" % self.default_index
            )



import os.path
import shutil
import subprocess
import tempfile

import mock
import pytest

from insights.client import crypto


# GPG2 exports keys in a way gpg1.4 cannot read. Since gpg1.4 is present in the
# Ubuntu image that is used in CI to test Python 2.6, we have to use the old
# version, so it is recognised by all CI branches.
#
# $ gpg --gen-key            # gpg1.x
# $ gpg --full-generate-key  # gpg2.x
# $ gpg --list-secret-keys --keyid-format long
# $ KEYID=xxx (line `sec`, after the `/` character)
# $ gpg --armor --export $KEYID
# $ gpg --armor --export-secret-keys $KEYID

GPG_PUBLIC_KEY = """-----BEGIN PGP PUBLIC KEY BLOCK-----
Version: GnuPG v1

mQENBGUlPEcBCACXhinLd4KiyQ9CWXM+gpgo0HMuTdESTlVVDYRjm/ebazephXq7
0hhAhXallAQJkaXFPuLETumwFYPx60agUDWsUie8gk3TLE+ejuTxoda0Yo6rehsD
zds1ptHQLzar00SFUlfiJXnMaobXLeNjDSglkynQC4uQODouDa1jkSRfNbDJOJYd
IBbPBIaJ3PlNrdVutPcx4yIZM1siqMZ9k0g2iYPWyp0ceuP0jpCTPc5TRRP5pLuz
INRJzhfJ+oPtbFt8Y4s1xEZ0Kfy0sA8Awk3VJqzoCU3ILBU5cGquQDdNggxTRSPV
X5Z4nW2zjN9lXAkwr29NL1rA1jjLt+c5upYZABEBAAG0SGluc2lnaHRzLWNvcmUg
KFNpZ25pbmcga2V5IGZvciB1bml0IHRlc3RpbmcpIDxpbnNpZ2h0cy1jb3JlQGV4
YW1wbGUub3JnPokBOAQTAQIAIgUCZSU8RwIbAwYLCQgHAwIGFQgCCQoLBBYCAwEC
HgECF4AACgkQn+6Vu126uXAHdgf/Q8a7Jruhyn+EDLL94gAc6kXubvVArVe9Rdpo
HwG7cj8wUa/7zd7FUcYJuz6bbebgmmlRwFf1CeodGURFpfwf1dkiKV5QqobeXhRp
srrkeLlMR8bZVFSCvU+oOETpJfMo6feDI/tQrOcIpxrd/cEu2XSQ1JBM/+8NbQiU
Ma1cWHOmQJ1OD7jWOllUq5hDs1vtUzPORUsYe1V1Dcx89gdlhfc1cc30yoLzDNqu
0Abn34CAthUkr0sWplzltXOY+Za1kOmQVlaLQ0W3tq8BDi79bdHpTtD/g9QO++Nc
r5xdP+tPNWAOUlyi7S6qscDLhMWv7o4eLq6UL9eUC/M2CWJSKg==
=9pSV
-----END PGP PUBLIC KEY BLOCK-----
"""

GPG_PRIVATE_KEY = """-----BEGIN PGP PRIVATE KEY BLOCK-----
Version: GnuPG v1

lQOYBGUlPEcBCACXhinLd4KiyQ9CWXM+gpgo0HMuTdESTlVVDYRjm/ebazephXq7
0hhAhXallAQJkaXFPuLETumwFYPx60agUDWsUie8gk3TLE+ejuTxoda0Yo6rehsD
zds1ptHQLzar00SFUlfiJXnMaobXLeNjDSglkynQC4uQODouDa1jkSRfNbDJOJYd
IBbPBIaJ3PlNrdVutPcx4yIZM1siqMZ9k0g2iYPWyp0ceuP0jpCTPc5TRRP5pLuz
INRJzhfJ+oPtbFt8Y4s1xEZ0Kfy0sA8Awk3VJqzoCU3ILBU5cGquQDdNggxTRSPV
X5Z4nW2zjN9lXAkwr29NL1rA1jjLt+c5upYZABEBAAEAB/0QIdsaTA+PCEwFGuPv
sFTF56eTsvpC8i8YnpdNSaI7nFcxR8JQ8+XcHLmMmG0znZuiG/dlwicUNb42CAAd
ely0i4yqf88MYCfb8EfEyB/FVcbtz9LHfWfM1wV4nkY6VgRyE1nC/I1yq5bOmxae
CZ0QHxJxEYGa6bmcBJ3Ev4O5VSKZRRByPI0HXmVB6GoqVtmwa7TFrlgLM5GPbgVe
P76lNY2me8jEnHqzrPpuCB/N0VSCEUf45RV+TNjWL1lpF8JPzXS8oGoxcDoo+sIQ
AdcfMFrRtf3DW8kJRZC8i4T9++/k3Sjak9GE8ocekCDJkFPRkKv3A4T8nvmMKbNx
fugZBADAGD+u4Uhwlyzvr84wmwIPcozhQhTWot8dom6LCiZ47sfyG1qVhvxqDKtc
ttPv+VfA0RyJb9PUKbOf71hA7lMm0qHyX6KTknYYl8u+s/ra8VSwdNyl2/DQDf2j
OjnIMLM9IC6TMQ/mvpvRmXLc6m6HfL6ZBAbAL0EwOMUAUelnBQQAye626NZJTZ3E
mQ9z6l6UEi3IZIw+uJfSg/HL4MgU4zRNsOqyJyizdmVzQm0h+1aHQu1SuFsllPXF
DOKR/IlkjQJCbiU1wZASYYJuwmmVdG4HqZD0bjKU40fJgzBJhOfnhi079jnZzMsL
+NltcmAtUFKteWsPZe7Blic4QmIbtwUEAIick2Ai788yKGrAry5XBwHNcSIYKNr2
FFWwjFQWAK82hG2tiKCtF8vg58KBvGR3KNUlCLqDeP85jK7+IuaExSlWKYxIKo70
8Vfpqpyol/+V1yZ2duEcPSXtrMTg5Yl/7PITyGPnM4mFjvwg+cuYRDqYTT0kWbrl
WxtODNwtZdIURTm0SGluc2lnaHRzLWNvcmUgKFNpZ25pbmcga2V5IGZvciB1bml0
IHRlc3RpbmcpIDxpbnNpZ2h0cy1jb3JlQGV4YW1wbGUub3JnPokBOAQTAQIAIgUC
ZSU8RwIbAwYLCQgHAwIGFQgCCQoLBBYCAwECHgECF4AACgkQn+6Vu126uXAHdgf/
Q8a7Jruhyn+EDLL94gAc6kXubvVArVe9RdpoHwG7cj8wUa/7zd7FUcYJuz6bbebg
mmlRwFf1CeodGURFpfwf1dkiKV5QqobeXhRpsrrkeLlMR8bZVFSCvU+oOETpJfMo
6feDI/tQrOcIpxrd/cEu2XSQ1JBM/+8NbQiUMa1cWHOmQJ1OD7jWOllUq5hDs1vt
UzPORUsYe1V1Dcx89gdlhfc1cc30yoLzDNqu0Abn34CAthUkr0sWplzltXOY+Za1
kOmQVlaLQ0W3tq8BDi79bdHpTtD/g9QO++Ncr5xdP+tPNWAOUlyi7S6qscDLhMWv
7o4eLq6UL9eUC/M2CWJSKg==
=ZQ6H
-----END PGP PRIVATE KEY BLOCK-----
"""

GPG_FINGERPRINT = "E884 7216 86A8 1EE3 EBF4  5771 9FEE 95BB 5DBA B970"
GPG_OWNER = (
    "insights-core (Signing key for unit testing) "
    "<insights-core@example.org>"
)


def _initialize_gpg_environment(home):
    """Save GPG keys and sign a file with them.

    The home directory is populated with the following files:
    - key.public.gpg
    - key.private.gpg
    - file.txt
    - file.txt.asc
    """
    # Save the public key into temporary file
    public_key = home + "/key.public.gpg"
    with open(public_key, "w") as f:
        f.write(GPG_PUBLIC_KEY)
    # It is strictly not necessary to import both public and private keys,
    #  the private key should be enough.
    #  However, the Python 2.6 CI image requires that.
    process = subprocess.Popen(
        ["/usr/bin/gpg", "--homedir", home, "--import", public_key],
        stdout=subprocess.PIPE, stderr=subprocess.PIPE,
        env={"LC_ALL": "C.UTF-8"},
    )
    process.communicate()
    assert process.returncode == 0

    # Save the private key into temporary file and import it
    private_key = home + "/key.private.gpg"
    with open(private_key, "w") as f:
        f.write(GPG_PRIVATE_KEY)
    process = subprocess.Popen(
        ["/usr/bin/gpg", "--homedir", home, "--import", private_key],
        stdout=subprocess.PIPE, stderr=subprocess.PIPE,
        env={"LC_ALL": "C.UTF-8"},
    )
    process.communicate()
    assert process.returncode == 0

    # Create a file and sign it
    file = home + "/file.txt"
    with open(file, "w") as f:
        f.write("a signed message")
    process = subprocess.Popen(
        ["/usr/bin/gpg", "--homedir", home, "--detach-sign", "--armor", file],
        stdout=subprocess.PIPE, stderr=subprocess.PIPE,
        env={"LC_ALL": "C.UTF-8"},
    )
    process.communicate()
    assert process.returncode == 0

    # Ensure the signature has been created
    assert os.path.exists(home + "/file.txt.asc")


@mock.patch("insights.client.crypto.GPGCommand.TEMPORARY_GPG_HOME_PARENT_DIRECTORY", "/tmp/")
def test_valid_signature():
    """A detached file signature can be verified."""
    home = tempfile.mkdtemp()
    _initialize_gpg_environment(home)

    # Run the test
    result = crypto.verify_gpg_signed_file(
        file=home + "/file.txt",
        signature=home + "/file.txt.asc",
        key=home + "/key.public.gpg",
    )
    shutil.rmtree(home, ignore_errors=True)

    # Verify results
    assert True is result.ok
    assert "" == result.stdout
    assert (
        'gpg: Good signature from "{owner}"'.format(owner=GPG_OWNER)
    ) in result.stderr
    assert (
        'Primary key fingerprint: {fp}'.format(fp=GPG_FINGERPRINT)
    ) in result.stderr
    assert 0 == result.return_code

    assert not os.path.isfile(result._command._home)


@pytest.mark.parametrize("file", ("/file.txt", "/file.txt.asc"))
def test_no_file(file):
    """Missing file or its signature is caught and returned as command result."""
    home = tempfile.mkdtemp()
    _initialize_gpg_environment(home)

    os.remove(home + file)

    result = crypto.verify_gpg_signed_file(
        file=home + "/file.txt",
        signature=home + "/file.txt.asc",
        key=home + "/key.public.gpg",
    )

    assert not os.path.isfile(home + file)
    assert result.ok is False
    assert result.return_code > 0
    assert "file '{path}' does not exist".format(path=home + file) == result.stderr

    shutil.rmtree(home, ignore_errors=True)


@mock.patch("insights.client.crypto.GPGCommand.TEMPORARY_GPG_HOME_PARENT_DIRECTORY", "/tmp/")
def test_invalid_public_key():
    """Invalid key is rejected."""
    home = tempfile.mkdtemp()
    _initialize_gpg_environment(home)

    with open(home + "/key.public.gpg", "w") as f:
        f.write("instead of the key, this file contains garbage")

    result = crypto.verify_gpg_signed_file(
        file=home + "/file.txt",
        signature=home + "/file.txt.asc",
        key=home + "/key.public.gpg",
    )

    assert result.ok is False
    assert result.return_code > 0


@mock.patch("insights.client.crypto.GPGCommand.TEMPORARY_GPG_HOME_PARENT_DIRECTORY", "/tmp/")
def test_invalid_signature():
    """A bad detached file signature can be detected."""
    home = tempfile.mkdtemp()
    _initialize_gpg_environment(home)

    # Change the contents of the file, making the signature incorrect
    with open(home + "/file.txt", "w") as f:
        f.write("an unsigned message")

    # Run the test
    result = crypto.verify_gpg_signed_file(
        file=home + "/file.txt",
        signature=home + "/file.txt.asc",
        key=home + "/key.public.gpg",
    )
    shutil.rmtree(home, ignore_errors=True)

    # Verify results
    assert not result.ok
    assert "" == result.stdout
    assert (
        'gpg: BAD signature from "{owner}"'.format(owner=GPG_OWNER)
    ) in result.stderr
    assert (
        'Primary key fingerprint: {fp}'.format(fp=GPG_FINGERPRINT)
    ) not in result.stderr
    assert 1 == result.return_code

    assert not os.path.isfile(result._command._home)


GPG_VERSION_CENTOS7 = """gpg (GnuPG) 2.0.22
libgcrypt 1.5.3
Copyright (C) 2013 Free Software Foundation, Inc.
...
"""
GPG_VERSION_CENTOS8 = """gpg (GnuPG) 2.2.20
libgcrypt 1.8.5
Copyright (C) 2020 Free Software Foundation, Inc.
...
"""
GPG_VERSION_CENTOS9 = """gpg (GnuPG) 2.3.3
libgcrypt 1.10.0-unknown
Copyright (C) 2021 Free Software Foundation, Inc.
...
"""


@pytest.mark.parametrize("output,supports", [
    (GPG_VERSION_CENTOS7, False),
    (GPG_VERSION_CENTOS8, True),
    (GPG_VERSION_CENTOS9, True),
    ("garbage output", False),
    ("gpg (GnuPG) 2.8", False),
    ("gpg (GnuPG) 2.8.1a9", False),
])
@mock.patch("insights.client.crypto.GPGCommand.TEMPORARY_GPG_HOME_PARENT_DIRECTORY", "/tmp/")
def test_gpg_socket_cleanup_support(output, supports):
    home = tempfile.mkdtemp()

    command = crypto.GPGCommand(command=[], key="")
    command._home = home

    class MockPopen:
        def __init__(self, stdout):
            self.stdout = stdout
            self.returncode = 0

        def communicate(self, *args, **kwargs):
            return self.stdout, ""

    with mock.patch("insights.client.crypto.subprocess.Popen", return_value=MockPopen(stdout=output)):
        result = command._supports_cleanup_socket()

    assert result == supports


@mock.patch("insights.client.crypto.GPGCommand.TEMPORARY_GPG_HOME_PARENT_DIRECTORY", "/tmp/")
@mock.patch("subprocess.Popen")
@mock.patch.object(crypto.GPGCommand, "_cleanup", return_value=None)
def test_invalid_gpg_setup(mock_cleanup, mock_popen):
    """An invalid GPG setup can be detected."""
    gpg_command = crypto.GPGCommand(command=[], key=os.path.join("/dummy", "key"))

    # Mock the process
    mock_process = mock.Mock()
    mock_process.communicate.return_value = (b"", b"gpg setup failed")
    mock_process.returncode = 1
    mock_popen.return_value = mock_process

    # Run the test
    result = gpg_command.evaluate()  # type: crypto.GPGCommandResult

    # Verify the results
    assert not result.ok
    assert "" == result.stdout
    assert "gpg setup failed" in result.stderr
    assert 1 == result.return_code



import doctest
import pytest
from insights.parsers import lsinitrd
from insights.parsers.lsinitrd import LsinitrdKdumpImage
from insights.tests import context_wrap


LSINITRD_ALL = """
Image: /boot/initramfs-3.10.0-862.el7.x86_64.img: 24M
========================================================================
Early CPIO image
========================================================================
drwxr-xr-x   3 root     root            0 Apr 20 15:58 .
-rw-r--r--   1 root     root            2 Apr 20 15:58 early_cpio
drwxr-xr-x   3 root     root            0 Apr 20 15:58 kernel
drwxr-xr-x   3 root     root            0 Apr 20 15:58 kernel/x86
drwxr-xr-x   2 root     root            0 Apr 20 15:58 kernel/x86/microcode
-rw-r--r--   1 root     root        12684 Apr 20 15:58 kernel/x86/microcode/AuthenticAMD.bin
========================================================================
Version: dracut-033-535.el7

Arguments: -f

dracut modules:
bash
nss-softokn
i18n
network
ifcfg
drm
plymouth
dm
kernel-modules
lvm
resume
rootfs-block
terminfo
udev-rules
biosdevname
systemd
usrmount
base
fs-lib
shutdown
========================================================================
drwxr-xr-x  12 root     root            0 Apr 20 15:58 .
crw-r--r--   1 root     root       5,   1 Apr 20 15:57 dev/console
crw-r--r--   1 root     root       1,  11 Apr 20 15:57 dev/kmsg
crw-r--r--   1 root     root       1,   3 Apr 20 15:57 dev/null
lrwxrwxrwx   1 root     root            7 Apr 20 15:57 bin -> usr/bin
========================================================================
""".strip()

LSINITRD_FILTERED = """
drwxr-xr-x   3 root     root            0 Apr 20 15:58 kernel/x86
Version: dracut-033-535.el7
dracut modules:
kernel-modules
udev-rules
drwxr-xr-x  12 root     root            0 Apr 20 15:58 .
crw-r--r--   1 root     root       5,   1 Apr 20 15:57 dev/console
crw-r--r--   1 root     root       1,  11 Apr 20 15:57 dev/kmsg
crw-r--r--   1 root     root       1,   3 Apr 20 15:57 dev/null
""".strip()

LSINITRD_EMPTY = ""

# This test case
LSINITRD_BROKEN = """
drwxr-xr-x   3 root     root            0 Apr 20 15:58 kernel/x86
Version: dracut-033-535.el7
dracut modules:
kernel-modules
udev-rules
drwxr-xr-x  12 root     root            0 Apr 20 15:58 .
crw-r--r--   1
crw-r-
c
""".strip()

LSINITRD_KDUMP_IMAGE_VALID = """
Image: initramfs-4.18.0-240.el8.x86_64kdump.img: 19M
========================================================================
Version: dracut-049-95.git20200804.el8

Arguments: --quiet --hostonly --hostonly-cmdline --hostonly-i18n --hostonly-mode 'strict' -o 'plymouth dash resume ifcfg earlykdump' --mount '/dev/mapper/rhel-root /sysroot xfs rw,relatime,seclabel,attr2,inode64,logbufs=8,logbsize=32k,noquota,nofail,x-systemd.before=initrd-fs.target' --no-hostonly-default-device -f

dracut modules:
bash
systemd
systemd-initrd
nss-softokn
rngd
i18n
drm
prefixdevname
dm
kernel-modules
kernel-modules-extra
lvm
qemu
fstab-sys
rootfs-block
terminfo
udev-rules
biosdevname
dracut-systemd
usrmount
base
fs-lib
kdumpbase
memstrack
microcode_ctl-fw_dir_override
shutdown
squash
========================================================================
crw-r--r--   1 root     root       5,   1 Aug  4  2020 dev/console
drwxr-xr-x   2 root     root            0 Aug  4  2020 run
-rw-r--r--   1 root     root          306 Aug  4  2020 usr/lib/dracut/build-parameter.txt
-rw-r--r--   1 root     root           30 Aug  4  2020 usr/lib/dracut/dracut-049-95.git20200804.el8
drwxr-xr-x   2 root     root            0 Aug  4  2020 usr/lib/dracut/hooks
-rw-r--r--   1 root     root          653 Aug  4  2020 usr/lib/dracut/hostonly-files
-rw-r--r--   1 root     root          780 Aug  4  2020 usr/lib/dracut/loaded-kernel-modules.txt
-rw-r--r--   1 root     root          273 Aug  4  2020 usr/lib/dracut/modules.txt
-rw-r--r--   1 root     root            0 Aug  4  2020 usr/lib/dracut/need-initqueue
-rw-r--r--   1 root     root            0 Aug  4  2020 usr/lib/dracut/no-switch-root
-rw-r--r--   1 root     root          221 Aug  4  2020 usr/lib/initrd-release
drwxr-xr-x   3 root     root            0 Aug  4  2020 usr/lib/modules
drwxr-xr-x   3 root     root            0 Aug  4  2020 usr/lib/modules/4.18.0-240.el8.x86_64
drwxr-xr-x   4 root     root            0 Aug  4  2020 usr/lib/modules/4.18.0-240.el8.x86_64/kernel
drwxr-xr-x   3 root     root            0 Aug  4  2020 usr/lib/modules/4.18.0-240.el8.x86_64/kernel/drivers
drwxr-xr-x   2 root     root            0 Aug  4  2020 usr/lib/modules/4.18.0-240.el8.x86_64/kernel/drivers/block
-rw-r--r--   1 root     root        16868 Aug  4  2020 usr/lib/modules/4.18.0-240.el8.x86_64/kernel/drivers/block/loop.ko.xz
drwxr-xr-x   4 root     root            0 Aug  4  2020 usr/lib/modules/4.18.0-240.el8.x86_64/kernel/fs
drwxr-xr-x   2 root     root            0 Aug  4  2020 usr/lib/modules/4.18.0-240.el8.x86_64/kernel/fs/overlayfs
-rw-r--r--   1 root     root        53736 Aug  4  2020 usr/lib/modules/4.18.0-240.el8.x86_64/kernel/fs/overlayfs/overlay.ko.xz
drwxr-xr-x   2 root     root            0 Aug  4  2020 usr/lib/modules/4.18.0-240.el8.x86_64/kernel/fs/squashfs
-rw-r--r--   1 root     root        24412 Aug  4  2020 usr/lib/modules/4.18.0-240.el8.x86_64/kernel/fs/squashfs/squashfs.ko.xz
-rw-r--r--   1 root     root        21985 Aug  4  2020 usr/lib/modules/4.18.0-240.el8.x86_64/modules.alias
-rw-r--r--   1 root     root        20181 Aug  4  2020 usr/lib/modules/4.18.0-240.el8.x86_64/modules.alias.bin
-rw-r--r--   1 root     root         7528 Aug  4  2020 usr/lib/modules/4.18.0-240.el8.x86_64/modules.builtin
-rw-r--r--   1 root     root         9683 Aug  4  2020 usr/lib/modules/4.18.0-240.el8.x86_64/modules.builtin.bin
-rw-r--r--   1 root     root         3632 Aug  4  2020 usr/lib/modules/4.18.0-240.el8.x86_64/modules.dep
-rw-r--r--   1 root     root         5304 Aug  4  2020 usr/lib/modules/4.18.0-240.el8.x86_64/modules.dep.bin
-rw-r--r--   1 root     root          126 Aug  4  2020 usr/lib/modules/4.18.0-240.el8.x86_64/modules.devname
-rw-r--r--   1 root     root       101785 Aug  4  2020 usr/lib/modules/4.18.0-240.el8.x86_64/modules.order
-rw-r--r--   1 root     root           85 Aug  4  2020 usr/lib/modules/4.18.0-240.el8.x86_64/modules.softdep
-rw-r--r--   1 root     root        50083 Aug  4  2020 usr/lib/modules/4.18.0-240.el8.x86_64/modules.symbols
-rw-r--r--   1 root     root        56186 Aug  4  2020 usr/lib/modules/4.18.0-240.el8.x86_64/modules.symbols.bin
drwxr-xr-x   2 root     root            0 Aug  4  2020 usr/lib64
drwxr-xr-x   2 root     root            0 Aug  4  2020 var/tmp
========================================================================
""".strip()

LSINITRD_KDUMP_IMAGE_VALID_EXAMPLE = """
Image: initramfs-4.18.0-240.el8.x86_64kdump.img: 19M
========================================================================
Version: dracut-049-95.git20200804.el8

Arguments: --quiet --hostonly --hostonly-cmdline --hostonly-i18n --hostonly-mode 'strict' -o 'plymouth dash resume ifcfg earlykdump' --mount '/dev/mapper/rhel-root /sysroot xfs rw,relatime,seclabel,attr2,inode64,logbufs=8,logbsize=32k,noquota,nofail,x-systemd.before=initrd-fs.target' --no-hostonly-default-device -f

dracut modules:
bash
systemd
systemd-initrd
i18n
========================================================================
crw-r--r--   1 root     root       5,   1 Aug  4  2020 dev/console
crw-r--r--   1 root     root       1,  11 Aug  4  2020 dev/kmsg
crw-r--r--   1 root     root       1,   3 Aug  4  2020 dev/null
crw-r--r--   1 root     root       1,   8 Aug  4  2020 dev/random
crw-r--r--   1 root     root       1,   9 Aug  4  2020 dev/urandom
drwxr-xr-x  14 root     root            0 Aug  4  2020 .
lrwxrwxrwx   1 root     root            7 Aug  4  2020 bin -> usr/bin
drwxr-xr-x   2 root     root            0 Aug  4  2020 dev
========================================================================
""".strip()

LSINITRD_LVM_CONF = """
# volume_list = [ "vg1", "vg2/lvol1", "@tag1", "@*" ]
volume_list = [ "vg2", "vg3/lvol3", "@tag2", "@*" ]
""".strip()


def test_lsinitrd_empty():
    d = lsinitrd.Lsinitrd(context_wrap(LSINITRD_EMPTY))
    assert len(d.data) == 0
    assert d.search(name__contains='kernel') == []
    assert d.unparsed_lines == []


def test_lsinitrd_filtered():
    d = lsinitrd.Lsinitrd(context_wrap(LSINITRD_FILTERED))
    assert len(d.data) == 5
    assert d.search(name__contains='kernel') == [{'type': 'd', 'perms': 'rwxr-xr-x', 'links': 3, 'owner': 'root', 'group': 'root', 'size': 0, 'date': 'Apr 20 15:58', 'name': 'kernel/x86', 'dir': ''}]
    assert d.unparsed_lines == ['Version: dracut-033-535.el7', 'dracut modules:', 'kernel-modules', 'udev-rules']


def test_lsinitrd_all():
    d = lsinitrd.Lsinitrd(context_wrap(LSINITRD_ALL))
    lsdev = d.search(name__contains='dev')
    assert len(lsdev) == 3
    dev_console = {
        'type': 'c', 'perms': 'rw-r--r--', 'links': 1, 'owner': 'root', 'group': 'root',
        'major': 5, 'minor': 1, 'date': 'Apr 20 15:57', 'name': 'dev/console', 'dir': '',
    }
    assert dev_console in lsdev
    assert 'dev/kmsg' in [l['name'] for l in lsdev]
    assert 'dev/null' in [l['name'] for l in lsdev]
    assert len(d.data) == 10
    assert len(d.unparsed_lines) == 32
    assert "udev-rules" in d.unparsed_lines


def test_lsinitrd_broken():
    """
    For this testcase, ls_parser.parse() will throw an IndexError.
    Assert with this specific error here.
    """
    with pytest.raises(Exception) as err:
        lsinitrd.Lsinitrd(context_wrap(LSINITRD_BROKEN))
    assert "list index out of range" in str(err)


def test_lsinitrd_kdump_image_valid():
    parser_result = LsinitrdKdumpImage(context_wrap(LSINITRD_KDUMP_IMAGE_VALID))
    assert parser_result is not None
    result_list = parser_result.search(name__contains='devname')
    assert len(result_list) == 1
    assert result_list[0].get('name') == 'usr/lib/modules/4.18.0-240.el8.x86_64/modules.devname'


def test_lsinitrd_lvm_conf():
    lvm_conf = lsinitrd.LsinitrdLvmConf(context_wrap(LSINITRD_LVM_CONF))
    assert lvm_conf["volume_list"] == ["vg2", "vg3/lvol3", "@tag2", "@*"]


def test_lsinitrd_docs():
    failed_count, tests = doctest.testmod(
        globs={'ls': lsinitrd.Lsinitrd(context_wrap(LSINITRD_FILTERED)),
               'lsinitrd_kdump_image': LsinitrdKdumpImage(context_wrap(LSINITRD_KDUMP_IMAGE_VALID_EXAMPLE)),
               'lsinitrd_lvm_conf': lsinitrd.LsinitrdLvmConf(context_wrap(LSINITRD_LVM_CONF))}
    )
    assert failed_count == 0



"""
LogrotateConfAll - Combiner for logrotate configuration
=======================================================
Combiner for accessing all the logrotate configuration files.  It collects all
LogrotateConf generated from each single logrotate configuration file.

There may be multiple logrotate configuration, and the main configuration file is
``/etc/logrotate.conf``.  Only the options defined in this file are global
options, and all other options (if there are) will be discarded.

"""
import operator
import os

from fnmatch import fnmatch
from insights.core import ConfigCombiner
from insights.core.plugins import combiner
from insights.parsers.logrotate_conf import LogrotateConf, LogRotateConfPEG
from insights.parsr.query import eq


@combiner(LogrotateConf)
class LogrotateConfAll(object):
    """
    Class for combining all the logrotate configuration files.

    Sample files::

        # /etc/logrotate.conf:
            compress
            rotate 7

            /var/log/messages {
                rotate 5
                weekly
                postrotate
                            /sbin/killall -HUP syslogd
                endscript
            }

        # /etc/logrotate.d/httpd
            "/var/log/httpd/access.log" /var/log/httpd/error.log {
                rotate 5
                mail www@my.org
                size=100k
                sharedscripts
                postrotate
                            /sbin/killall -HUP httpd
                endscript
            }

        # /etc/logrotate.d/newscrit
            /var/log/news/*.crit  {
                monthly
                rotate 2
                olddir /var/log/news/old
                missingok
                postrotate
                            kill -HUP `cat /var/run/inn.pid`
                endscript
                nocompress
            }

    Examples:
        >>> all_lrt.global_options
        ['compress', 'rotate', 'include']
        >>> all_lrt['rotate']
        '7'
        >>> '/var/log/httpd/access.log' in all_lrt.log_files
        True
        >>> all_lrt['/var/log/httpd/access.log']['rotate']
        '5'
        >>> all_lrt.configfile_of_logfile('/var/log/news/olds.crit')
        '/etc/logrotate.d/newscrit'
        >>> all_lrt.options_of_logfile('/var/log/httpd/access.log')['mail']
        'www@my.org'


    Attributes:
        data(dict): All parsed options and log files are stored in this
            dictionary
        global_options(list): List of global options defined in
            ``/etc/logrotate.conf``
        log_files(list): List of log files in all logrotate configuration files
    """
    def __init__(self, lrt_conf):
        self.data = {}
        self.global_options = []
        self.log_files = []
        self._file_map = {}
        self.include = None
        for lrt in lrt_conf:
            if lrt.file_path == "/etc/logrotate.conf":
                self.data.update(lrt.data)
                self.log_files.extend(lrt.log_files)
                self._file_map[lrt.file_path] = lrt.log_files
                self.global_options = lrt.options
                if 'include' in lrt.options and lrt.data['include'].startswith('/etc/logrotate.d'):
                    self.include = lrt.data['include']

        if self.include:
            for lrt in lrt_conf:
                if lrt.file_path == self.include or (lrt.file_path.startswith(self.include) and len(lrt.file_path.split(self.include)[-1].lstrip('/').split('/')) == 1) or fnmatch(lrt.file_path, self.include):
                    self.data.update(lrt.data)
                    self.log_files.extend(lrt.log_files)
                    self._file_map[lrt.file_path] = lrt.log_files

    def __contains__(self, item):
        return (item in self.global_options or
                any(fnmatch(item, f) for f in self.log_files))

    def __getitem__(self, item):
        if item in self.global_options:
            return self.data[item]
        return self.data.get(self._get_match_logfile(item))

    def _get_match_logfile(self, log_file, file_list=[]):
        file_list = self.log_files if not file_list else file_list
        for f in file_list:
            if fnmatch(log_file, f):
                return f

    def options_of_logfile(self, log_file):
        """
        Get the options of ``log_file``.

        Parameters:
            log_file(str): The log files need to check.

        Returns:
            dict: Dictionary contains the options of ``log_file``.  None when no
            such ``log_file``.
        """
        return self.data.get(self._get_match_logfile(log_file))

    def configfile_of_logfile(self, log_file):
        """
        Get the configuration file path in which the ``log_file`` is configured.

        Parameters:
            log_file(str): The log files need to check.

        Returns:
            dict: The configuration file path of ``log_file``.  None when no
            such ``log_file``.
        """
        for f, lfs in self._file_map.items():
            for file in lfs:
                if fnmatch(log_file, file):
                    return f


@combiner(LogRotateConfPEG)
class LogRotateConfTree(ConfigCombiner):
    """
    Exposes logrotate configuration through the parsr query interface.

    See the :py:class:`insights.core.ConfigComponent` class for example usage.
    """
    def __init__(self, confs):
        include = eq("include")
        main_file = "logrotate.conf"
        super(LogRotateConfTree, self).__init__(confs, main_file, include)

    @property
    def conf_path(self):
        return os.path.dirname(self.main.file_path)

    def find_matches(self, confs, pattern):
        results = []
        for c in confs:
            if c.file_path.startswith(pattern) and len(c.file_path.split(pattern)[-1].lstrip('/').split('/')) == 1:
                results.append(c)
            elif fnmatch(c.file_path, pattern):
                results.append(c)
        return sorted(results, key=operator.attrgetter("file_name"))



import socket
from insights.util.hostname import determine_hostname


def test_determine_hostname_display_name():
    assert determine_hostname(display_name='foo') == 'foo'


def test_determine_hostname():
    hostname = socket.gethostname()
    fqdn = socket.getfqdn()
    assert determine_hostname() in (hostname, fqdn)
    assert determine_hostname() != 'foo'



# coding=utf-8
"""
Yum - Commands
==============
Parsers for ``yum`` commands.

This module contains the classes that parse the output of the commands
`yum -C --noplugins repolist`.

YumRepoList - command ``yum -C --noplugins repolist``
-----------------------------------------------------
"""
from insights.core import CommandParser
from insights.core.exceptions import ParseException, SkipComponent
from insights.core.plugins import parser
from insights.parsers import parse_fixed_table
from insights.specs import Specs

eus = [
    '5.0.z',
    '5.1.z',
    '5.2.z',
    '5.3.ll',
    '5.3.z',
    '5.4.z',
    '5.6.ll',
    '5.6.z',
    '5.9.ll',
    '5.9.z'
    '6.0.z',
    '6.1.z',
    '6.2.aus',
    '6.2.z',
    '6.3.z',
    '6.4.aus',
    '6.4.z',
    '6.5.aus',
    '6.5.z',
    '6.6.aus',
    '6.6.z',
    '6.7.z'
]


@parser(Specs.yum_repolist)
class YumRepoList(CommandParser):
    """
    Class for parsing the output of `yum -C --noplugins repolist` command.

    Typical output of the command is::

        repo id                                             repo name                                                                                                    status
        rhel-7-server-e4s-rpms/x86_64                       Red Hat Enterprise Linux 7 Server - Update Services for SAP Solutions (RPMs)                                 12,250
        !rhel-ha-for-rhel-7-server-e4s-rpms/x86_64          Red Hat Enterprise Linux High Availability (for RHEL 7 Server) Update Services for SAP Solutions (RPMs)         272
        *rhel-sap-hana-for-rhel-7-server-e4s-rpms/x86_64    RHEL for SAP HANA (for RHEL 7 Server) Update Services for SAP Solutions (RPMs)                                   21
        repolist: 12,768

    Or sometimes it just outputs repo id and status::

        repo id                                                                               status
        LME_EPEL_6_x86_64                                                                        26123
        LME_FSMLabs_Timekeeper_timekeeper                                                            2
        LME_HP_-_Software_Delivery_Repository_Firmware_Pack_for_ProLiant_-_6Server_-_Current      1163
        LME_HP_-_Software_Delivery_Repository_Scripting_Took_Kit_-_6Server                          17
        LME_HP_-_Software_Delivery_Repository_Service_Pack_for_ProLiant_-_6Server_-_Current       1915
        LME_HP_-_Software_Delivery_Repository_Smart_Update_Manager_-_6Server                        30
        LME_LME_Custom_Product_Mellanox_OFED                                                       114
        LME_LME_Custom_Product_OMD_RPMS                                                             14
        LME_LME_Custom_Product_RPMs                                                                  5
        LME_LME_Custom_Product_SNOW_Repository                                                       2
        rhel-6-server-optional-rpms                                                            10400+1
        rhel-6-server-rpms                                                                    18256+12
        rhel-6-server-satellite-tools-6.2-rpms                                                      55
        repolist: 58096

    Examples:
        >>> len(repolist)
        3
        >>> 'rhel-7-server-e4s-rpms/x86_64' in repolist.repos
        True
        >>> 'rhel-7-server-e4s-rpms' in repolist.repos
        False
        >>> 'rhel-7-server-e4s-rpms' in repolist.rhel_repos
        True
        >>> repolist['rhel-7-server-e4s-rpms/x86_64']['name']
        'Red Hat Enterprise Linux 7 Server - Update Services for SAP Solutions (RPMs)'
        >>> repolist[0]['name']
        'Red Hat Enterprise Linux 7 Server - Update Services for SAP Solutions (RPMs)'
        >>> repolist['rhel-ha-for-rhel-7-server-e4s-rpms/x86_64']['id']
        '!rhel-ha-for-rhel-7-server-e4s-rpms/x86_64'
        >>> len(repolist_no_reponame)
        13
        >>> len(repolist_no_reponame.rhel_repos)
        3
        >>> 'rhel-6-server-rpms' in repolist_no_reponame.repos
        True
        >>> 'rhel-6-server-optional-rpms' in repolist_no_reponame.rhel_repos
        True
        >>> repolist_no_reponame[0]['id']
        'LME_EPEL_6_x86_64'
        >>> repolist_no_reponame[0].get('name', '')
        ''

    Attributes:
        data (list): list of repos wrapped in dictionaries
        repos (dict): dict of all listed repos where the key is the full repo-id without "!" or "*".
            But you can get it from the value part if needed. For example::

                self.repos = {
                    'rhel-7-server-e4s-rpms/x86_64': {
                        'id': 'rhel-7-server-e4s-rpms/x86_64',
                        'name': 'Red Hat Enterprise Linux 7 Server - Update Services for SAP Solutions (RPMs)',
                        'status': '12,250'
                    },
                    'rhel-ha-for-rhel-7-server-e4s-rpms/x86_64': {
                        'id': '!rhel-ha-for-rhel-7-server-e4s-rpms/x86_64',
                        'name': 'Red Hat Enterprise Linux High Availability (for RHEL 7 Server) Update Services for SAP Solutions (RPMs)',
                        'status': '272'
                    },
                    'rhel-sap-hana-for-rhel-7-server-e4s-rpms/x86_64': {
                        'id': '*rhel-sap-hana-for-rhel-7-server-e4s-rpms/x86_64',
                        'name': 'RHEL for SAP HANA (for RHEL 7 Server) Update Services for SAP Solutions (RPMs)',
                        'status': '21'
                    }
                }
    """
    _no_repo_available_msg_strs = [
        "No repositories available",
        "No hay ningún repositorio disponible",
        "Keine Paketquellen verfügbar",
        "Nessun repository disponibile",
        "Ez dago biltegirik erabilgarri",
        "Walang repositories na magagamit",
        "Няма налични хранилища",
    ]
    _repo_headers = [
        # repo_id, repo_name, repo_id_length, repo_name_length
        ("repo id", "repo name", 7, 9),
        ("id du dépôt", "nom du dépôt", 11, 12),
        ("Paketquellen-ID", "Paketquellen-Name", 15, 17),
        ("id del repositorio", "nombre del repositorio", 18, 22),
        ("id do repo", "nome do repo", 10, 12),
        ("id repo", "nome repo", 7, 9),
        ("identyfikator repozytorium", "nazwa repozytorium", 26, 18),
        ("tároló azonosító", "tároló neve", 16, 11),
        ("Paketquellenkennung", "Paketquellenname", 19, 16),
        ("仓库 id", "仓库名称", 7, 4),
        ("ід. сховища", "назва сховища", 11, 13),
    ]

    def parse_content(self, content):
        if not content:
            raise SkipComponent('No repolist.')

        if content[0].startswith('repolist:'):
            raise SkipComponent('No repolist.')

        for no_repo_available_msg in self._no_repo_available_msg_strs:
            if content[0].startswith(no_repo_available_msg):
                raise SkipComponent('No repolist.')

        trailing_line_prefix = [
                'repolist:',
                'Uploading Enabled',
                'Loaded plugins:',
                'Plugin ',
                'Unable to upload',
        ]

        self.data = []
        self.repos = {}

        # Header table language defaults to english: self._repo_headers[0]
        # Give it a try for table header in other languages when the default one is not found
        error_raised_for_stander_header_repo_id = None
        for header_pair_index, (header_repo_id, header_repo_name, repo_id_len, repo_name_len) in enumerate(self._repo_headers):
            try:
                self.data = parse_fixed_table(
                        content,
                        heading_ignore=[header_repo_id],
                        header_substitute=[(header_repo_id, 'id' + ' ' * (repo_id_len - 2)),
                                            (header_repo_name, 'name' + ' ' * (repo_name_len - 4))],
                        trailing_ignore=trailing_line_prefix,
                        empty_exception=True)
                break
            except ValueError as e:
                # ValueError raised by parse_fixed_table
                # Likely be a unrecognized header_repo_id raised by calc_offset
                if header_pair_index == 0:
                    error_raised_for_stander_header_repo_id = e
                # When the last pair of table header trying fails, end the parsing by raising a error
                if header_pair_index == len(self._repo_headers) - 1:
                    raise ParseException('Failed to parser yum repolist: {0}'.format(
                                    str(error_raised_for_stander_header_repo_id or e)))

        if not self.data:
            raise SkipComponent('No repolist.')

        self.repos = dict((d['id'].lstrip('!').lstrip('*'), d) for d in self.data)

    def __len__(self):
        return len(self.data)

    def __getitem__(self, idx):
        if isinstance(idx, int):
            return self.data[idx]
        elif isinstance(idx, str):
            return self.repos[idx]

    def __contains__(self, _id):
        return _id in self.repos

    @property
    def eus(self):
        '''list of the EUS part of each repo'''
        euses = []
        for repo in [r["id"] for r in self.data]:
            if repo.startswith("rhel-") and "server-" in repo:
                _, eus_version = repo.split("server-", 1)
                if eus_version in eus:
                    euses.append(eus_version)
        return euses

    @property
    def rhel_repos(self):
        """
        list: list of all the rhel repos and the item is just the repo id without server and arch info.
            For example::

                self.rhel_repos = [
                    'rhel-7-server-e4s-rpms',
                    'rhel-ha-for-rhel-7-server-e4s-rpms',
                    'rhel-sap-hana-for-rhel-7-server-e4s-rpms'
                ]
        """
        return [i.split('/')[0]
                for i in self.repos
                if i.startswith('rhel-') or '-rhel-' in i]



import os
import pytest
from mock.mock import patch
from insights.client.constants import InsightsConstants as constants
from insights.client.connection import InsightsConnection
from insights.client.config import InsightsConfig


@patch("insights.client.connection.InsightsConnection._init_session")
@patch('insights.client.utilities.generate_machine_id')
@patch('insights.client.connection.InsightsConnection.get')
@patch('insights.client.connection.os.path.isfile', return_value=False)
def test_unregistered_check_results(_is_file, get, generate_machine_id, _init_session):
    config = InsightsConfig()
    connection = InsightsConnection(config)

    with pytest.raises(Exception):
        connection.get_advisor_report()

    assert not os.path.isfile(constants.registered_files[0])
    generate_machine_id.assert_not_called()
    get.assert_not_called()



import os
import shlex
import subprocess
import tempfile
import zipfile
from contextlib import closing

from insights.core.hydration import get_all_files
from insights.core.archives import extract


def test_with_zip():
    tmp_dir = tempfile.mkdtemp()

    d = os.path.join(tmp_dir, 'sys', 'kernel')
    os.makedirs(d)
    with open(os.path.join(d, 'kexec_crash_size'), "w") as f:
        f.write("ohyeahbaby")

    try:
        os.unlink("/tmp/test.zip")
    except:
        pass

    # stolen from zipfile.py:main
    def _add_to_zip(zf, path, zippath):
        if os.path.isfile(path):
            zf.write(path, zippath, zipfile.ZIP_DEFLATED)
        elif os.path.isdir(path):
            if zippath:
                zf.write(path, zippath)
            for nm in os.listdir(path):
                _add_to_zip(zf, os.path.join(path, nm), os.path.join(zippath, nm))
        # else: ignore

    with closing(zipfile.ZipFile("/tmp/test.zip", "w")) as zf:
        _add_to_zip(zf, tmp_dir, os.path.basename(tmp_dir))

    try:
        with extract("/tmp/test.zip") as ex:
            assert any(
                f.endswith("/sys/kernel/kexec_crash_size") for f in get_all_files(ex.tmp_dir)
            )

    finally:
        os.unlink("/tmp/test.zip")

    subprocess.call(shlex.split("rm -rf %s" % tmp_dir))



"""
KubeletConf file ``/etc/kubernetes/kubelet.conf``
=================================================
This parser is used to parse ``/etc/kubernetes/kubelet.conf
file information from sosreport.
"""
from insights.core.plugins import parser
from insights.core import JSONParser
from insights.specs import Specs


@parser(Specs.kubelet_conf)
class KubeletConf(JSONParser):
    """Class to parse ``/etc/kubernetes/kubelet.conf`` file"""



# -*- coding: utf-8 -*-
import os

from mock.mock import patch

from insights.client.archive import InsightsArchive
from insights.client.config import InsightsConfig
from insights.cleaner import Cleaner


def test_clean_file_obfuscate():
    conf = InsightsConfig(obfuscate=True)
    arch = InsightsArchive(conf)
    arch.create_archive_dir()

    # netstat_-neopa
    line = "tcp6       0      0 10.0.0.1:23           10.0.0.110:63564   ESTABLISHED 0"
    ret = "tcp6       0      0 10.230.230.2:23       10.230.230.1:63564 ESTABLISHED 0"

    test_dir = os.path.join(arch.archive_dir, 'data', 'etc')
    os.makedirs(test_dir)
    pp = Cleaner(conf, {})

    # netstat
    test_file = os.path.join(arch.archive_dir, 'data', 'testfile.netstat_-neopa')
    with open(test_file, 'w') as t:
        t.write(line)
    pp.clean_file(test_file, no_obfuscate=[])
    # file is changed per netstat logic
    with open(test_file, 'r') as t:
        assert ret == ''.join(t.readlines())

    arch.delete_archive_dir()


def test_clean_file_obfuscate_disabled_by_no_obfuscate():
    conf = InsightsConfig(obfuscate=True)
    arch = InsightsArchive(conf)
    arch.create_archive_dir()

    # netstat_-neopa
    line = "tcp6       0      0 10.0.0.1:23           10.0.0.110:63564   ESTABLISHED 0"

    test_dir = os.path.join(arch.archive_dir, 'data', 'etc')
    os.makedirs(test_dir)
    pp = Cleaner(conf, {})

    # netstat
    test_file = os.path.join(arch.archive_dir, 'data', 'testfile.netstat_-neopa')
    with open(test_file, 'w') as t:
        t.write(line)
    pp.clean_file(test_file, no_obfuscate=['ipv4'])
    # file is NOT changed
    with open(test_file, 'r') as t:
        assert line == ''.join(t.readlines())

    arch.delete_archive_dir()


@patch("insights.cleaner.Cleaner.clean_content")
def test_clean_file_non_exist(func):
    conf = InsightsConfig(obfuscate=True)
    arch = InsightsArchive(conf)
    arch.create_archive_dir()

    test_dir = os.path.join(arch.archive_dir, 'data', 'etc')
    os.makedirs(test_dir)
    pp = Cleaner(conf, {})

    pp.clean_file('non_existing_file', no_obfuscate=[])
    func.assert_not_called()

    # empty file
    test_file = os.path.join(arch.archive_dir, 'data', 'etc', 'x.conf')
    open(test_file, 'w').close()
    pp.clean_file(test_file, no_obfuscate=[])
    func.assert_called_once()

    arch.delete_archive_dir()



import pytest

from insights import SkipComponent
from insights.components.cryptsetup import HasCryptsetupWithTokens, HasCryptsetupWithoutTokens
from insights.parsers.installed_rpms import InstalledRpms
from insights.tests import context_wrap

RPMS_TOKENS_SUPPORT = "cryptsetup-2.4.3-2.fc36.x86_64"
RPMS_NO_TOKENS_SUPPORT = "cryptsetup-2.0.3-6.el7.x86_64"
RPMS_CRYPTSETUP_NOT_INSTALLED = "cryptsetup-libs-2.0.3-6.el7.x86_64"


def test_has_cryptsetup_with_tokens():
    rpms = InstalledRpms(context_wrap(RPMS_TOKENS_SUPPORT))
    HasCryptsetupWithTokens(rpms)

    rpms = InstalledRpms(context_wrap(RPMS_NO_TOKENS_SUPPORT))
    with pytest.raises(SkipComponent):
        HasCryptsetupWithTokens(rpms)

    rpms = InstalledRpms(context_wrap(RPMS_CRYPTSETUP_NOT_INSTALLED))
    with pytest.raises(SkipComponent):
        HasCryptsetupWithTokens(rpms)


def test_has_cryptsetup_without_tokens():
    rpms = InstalledRpms(context_wrap(RPMS_NO_TOKENS_SUPPORT))
    HasCryptsetupWithoutTokens(rpms)

    rpms = InstalledRpms(context_wrap(RPMS_TOKENS_SUPPORT))
    with pytest.raises(SkipComponent):
        HasCryptsetupWithoutTokens(rpms)

    rpms = InstalledRpms(context_wrap(RPMS_CRYPTSETUP_NOT_INSTALLED))
    with pytest.raises(SkipComponent):
        HasCryptsetupWithoutTokens(rpms)



from insights.parsr.query.boolean import TRUE, FALSE


def test_and():
    q = TRUE & TRUE
    assert q(None)

    q = TRUE & FALSE
    assert not q(None)

    q = FALSE & TRUE
    assert not q(None)

    q = FALSE & FALSE
    assert not q(None)


def test_or():
    q = TRUE | TRUE
    assert q(None)

    q = TRUE | FALSE
    assert q(None)

    q = FALSE | TRUE
    assert q(None)

    q = FALSE | FALSE
    assert not q(None)


def test_not():
    q = ~FALSE
    assert q(None)
    q = ~TRUE
    assert not q(None)
